package com.xxh.cloud.basic.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.client.api.bean.dto.VisitVerificationSearchDTO;
import com.xxh.cloud.basic.framework.modules.biz.service.VisitVerificationConfigBizService;
import com.xxh.cloud.basic.framework.modules.cache.service.VisitVerificationCacheService;
import com.xxh.cloud.framework.common.entity.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@SaCheckLogin
@RestController
@RequestMapping("/app/basic/ver")
@Api(tags="App_访问校验配置")
public class VisitVerificationAppController {

    @Autowired
    private VisitVerificationClientService visitVerificationClientService;
    @Autowired
    private VisitVerificationConfigBizService visitVerificationConfigBizService;
    @Autowired
    private VisitVerificationCacheService visitVerificationCacheService;

    @PostMapping("/syncPatientRecordBlackList")
    @ApiOperation(value="同步黑名单",response = Void.class)
    public Result<String> syncPatientRecordBlackList(@RequestBody Map<String, String> params) {
        visitVerificationConfigBizService.syncPatientRecordBlackList();
        return new Result<String>().success();
    }

    @PostMapping("/syncPatient4IdCard")
    @ApiOperation(value="根据身份证扩展患者ID",response = Void.class)
    public Result<String> syncPatient4IdCard(@RequestBody Map<String, String> params) {
        visitVerificationConfigBizService.syncPatient4IdCard();
        return new Result<String>().success();
    }

    @PostMapping("/cacheVisitVerification")
    @ApiOperation(value="缓存访问校验配置",response = Void.class)
    public Result<String> cacheVisitVerification(@RequestBody Map<String, String> params) {
        visitVerificationCacheService.cacheVisitVerification();
        return new Result<String>().success();
    }

    @PostMapping("/getVisitVerification")
    @ApiOperation(value="查询访问校验配置",response = VisitVerificationBO.class)
    public Result<Map<String, List<VisitVerificationBO>>> getVisitVerification(@RequestBody VisitVerificationSearchDTO params) {
        VisitVerificationSearchDTO dto = new VisitVerificationSearchDTO(params.getVisitType(), params.getVisitIds());
        return new Result<Map<String, List<VisitVerificationBO>>>().ok(visitVerificationClientService.getVisitVerification(dto));
    }

    @PostMapping("/getVisitVerificationByPatient")
    @ApiOperation(value="查询访问校验配置byPatientId",response = VisitVerificationBO.class)
    public Result<Map<String, List<VisitVerificationBO>>> getVisitVerificationByPatient(@RequestBody List<String> params) {
        return new Result<Map<String, List<VisitVerificationBO>>>().ok(visitVerificationClientService.getVisitVerificationByPatient(params));
    }
}
