package com.xxh.cloud.basic.framework.modules.base.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.DictData;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictDataVO;
import com.xxh.cloud.basic.framework.modules.biz.helper.DictConvertHelper;
import com.xxh.cloud.basic.framework.modules.base.mapper.DictDataMapper;
import com.xxh.cloud.basic.framework.modules.base.service.DictDataService;
import com.xxh.cloud.basic.framework.modules.cache.helper.CacheBasicKeyHelper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 字典数据表 服务实现类
 */
@Service
public class DictDataServiceImpl extends ServiceImpl<DictDataMapper, DictData> implements DictDataService {

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public List<DictDataVO> list(DictDataListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DictData> list= this.list(new QueryWrapper<DictData>()
                .eq(StrUtil.isNotEmpty(params.getDictTypeId()),"dict_type_id",params.getDictTypeId())
                .in(CollUtil.isNotEmpty(params.getDictTypeIds()),"dict_type_id",params.getDictTypeIds())
                .orderByDesc("dict_type_id","sort")
        );
        List<DictDataVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(DictConvertHelper.getDictDataVO(item));
            });
        }
        return data;
    }

    @Override
    public DictDataVO add(DictDataAddDTO params) {
        ValidatorUtils.validateEntity(params);
        long num=this.count(new QueryWrapper<DictData>()
                .eq("dict_type_id",params.getDictTypeId())
                .eq("dict_value",params.getDictValue())
        );
        if(num>0){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS);
        }
        DictData entity=new DictData();
        entity.setId(IdUtil.getSnowflake(3,3).nextIdStr());
        entity.setDictTypeId(params.getDictTypeId());
        entity.setDictLabel(params.getDictLabel());
        entity.setDictValue(params.getDictValue());
        entity.setRemark(params.getRemark());
        entity.setSort(params.getSort());
        entity.setStatus(params.getStatus());
        this.save(entity);
        return DictConvertHelper.getDictDataVO(entity);
    }

    @Override
    public DictDataVO update(DictDataUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DictData entity= this.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(!entity.getDictValue().equals(params.getDictValue())){
            long num=this.count(new QueryWrapper<DictData>()
                    .eq("dict_type_id",entity.getDictTypeId())
                    .eq("dict_value",params.getDictValue())
            );
            if(num>0){
                throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS);
            }
            entity.setDictValue(params.getDictValue());
        }

        entity.setDictLabel(params.getDictLabel());
        entity.setRemark(params.getRemark());
        entity.setSort(params.getSort());
        entity.setStatus(params.getStatus());
        this.updateById(entity);
        redisUtils.delete(CacheBasicKeyHelper.getDictCacheKey(entity.getDictTypeId()));
        return DictConvertHelper.getDictDataVO(entity);
    }

    @Override
    public void save2updateBatch(DictDataAdd2UpdateBatchDTO params) {
        ValidatorUtils.validateEntity(params);

        List<DictData> list=new ArrayList<>();
        for(DictDataAdd2UpdateItem item: params.getItems()){
            DictData entity=new DictData();
            entity.setId(IdUtil.getSnowflake(3,3).nextIdStr());
            entity.setDictTypeId(params.getDictTypeId());
            entity.setDictLabel(item.getDictLabel());
            entity.setDictValue(item.getDictValue());
            entity.setRemark(item.getRemark());
            entity.setStatus(item.getStatus());
            entity.setSort(item.getSort());
            list.add(entity);
        }
        this.remove(new QueryWrapper<DictData>().eq("dict_type_id",params.getDictTypeId()));
        this.saveBatch(list);
        redisUtils.delete(CacheBasicKeyHelper.getDictCacheKey(params.getDictTypeId()));
    }

    @Override
    public DictDataVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DictData entity= this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DictConvertHelper.getDictDataVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DictData entity= this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        this.removeById(id);
        redisUtils.delete(CacheBasicKeyHelper.getDictCacheKey(entity.getDictTypeId()));
    }
}
