package com.xxh.cloud.basic.framework.modules.base.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.DictData;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.DictType;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.DictTypeAddDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.DictTypeListDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.DictTypeQueryDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.DictTypeUpdateDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictTypeTreeVO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictTypeVO;
import com.xxh.cloud.basic.framework.modules.biz.helper.DictConvertHelper;
import com.xxh.cloud.basic.framework.modules.base.mapper.DictTypeMapper;
import com.xxh.cloud.basic.framework.modules.base.service.DictDataService;
import com.xxh.cloud.basic.framework.modules.base.service.DictTypeService;
import com.xxh.cloud.basic.framework.modules.cache.helper.CacheBasicKeyHelper;
import com.xxh.cloud.basic.framework.modules.constant.DictTypeCategory;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 字典类型 服务实现类
 * </p>
 */
@Service
public class DictTypeServiceImpl extends ServiceImpl<DictTypeMapper, DictType> implements DictTypeService {

    @Autowired
    private DictDataService dictDataService;
    @Autowired
    private RedisUtils redisUtils;

    @Override
    public PageData<DictTypeVO> query(DictTypeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<DictType> page= this.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DictType>()
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .orderByDesc("type","sort")
        );
        PageData<DictTypeVO> data= new PageConverter<DictType, DictTypeVO>() {
            @Override
            public DictTypeVO converter(DictType entity) {
                return DictConvertHelper.getDictTypeVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DictTypeVO> list(DictTypeListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DictType> list= this.list(new QueryWrapper<DictType>()
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .orderByDesc("type","sort")
        );

        List<DictTypeVO> data=new ArrayList<>();

        if(CollUtil.isNotEmpty(list)){
            Set<String> dictTypeIds=new HashSet<>();
            list.forEach(item->{
                dictTypeIds.add(item.getId());
            });

            List<DictData> dictDataList = dictDataService.list(
                    new QueryWrapper<DictData>()
                            .in("dict_type_id",dictTypeIds)
                            .orderByDesc("dict_type_id","sort")
            );

            list.forEach(item->{
                data.add(DictConvertHelper.getDictTypeVO(item,dictDataList));
            });
        }
        return data;
    }

    @Override
    public DictTypeVO add(DictTypeAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DictType entity=new DictType();
        entity.setId(params.getId());
        entity.setType(params.getType());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setSort(params.getSort());
        this.save(entity);
        return DictConvertHelper.getDictTypeVO(entity);
    }

    @Override
    public DictTypeVO update(DictTypeUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DictType entity= this.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setType(params.getType());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setSort(params.getSort());
        this.updateById(entity);
        redisUtils.delete(CacheBasicKeyHelper.getDictCacheKey(params.getId()));
        return DictConvertHelper.getDictTypeVO(entity);
    }


    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DictType entity= this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        this.removeById(id);
        dictDataService.remove(new QueryWrapper<DictData>().eq("dict_type_id",id));
        redisUtils.delete(CacheBasicKeyHelper.getDictCacheKey(id));
    }

    @Override
    public DictTypeVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        DictType entity= this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        List<DictData> dictDataList = dictDataService.list(
                new QueryWrapper<DictData>()
                        .eq("dict_type_id",id)
                        .orderByDesc("sort")
        );
        return DictConvertHelper.getDictTypeVO(entity,dictDataList);
    }

    @Override
    public DictTypeVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        DictType entity= this.getById(id);
        if(entity==null){
            return null;
        }
        List<DictData> dictDataList = dictDataService.list(
                new QueryWrapper<DictData>()
                        .eq("dict_type_id",id)
                        .orderByDesc("sort")
        );
        return DictConvertHelper.getDictTypeVO(entity,dictDataList);
    }

    @Override
    public List<DictTypeTreeVO> tree() {
        List<DictType> list= this.list(new QueryWrapper<DictType>()
                .orderByDesc("type","sort")
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<DictTypeTreeVO> treelist=new ArrayList<>();
        Set<String> types=new HashSet<>();
        list.forEach(item->{
            treelist.add(new DictTypeTreeVO(item.getId(),item.getName(),item.getType()));
            types.add(item.getType());
        });
        types.forEach(item->{
            DictTypeCategory dictTypeCategory= XXEnumUtil.getEnum(DictTypeCategory.class,item);
            treelist.add(new DictTypeTreeVO(dictTypeCategory.getCode(),dictTypeCategory.getName()));
        });
        return TreeUtils.build(treelist);
    }


}
