package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.framework.modules.biz.service.CompanyBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.Company;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.CompanyVO;
import com.xxh.cloud.basic.framework.modules.biz.helper.CompanyConvertHelper;
import com.xxh.cloud.basic.framework.modules.base.service.CompanyService;
import com.xxh.cloud.basic.framework.modules.cache.service.CompanyCacheService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Service
public class CompanyBizServiceImpl implements CompanyBizService {

    @Autowired
    private CompanyService companyService;
    @Autowired
    private CompanyCacheService companyCacheService;

    @Override
    public PageData<CompanyVO> query(CompanyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<Company> page= companyService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<Company>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getSysType()),"sys_type",params.getSysType())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .orderByDesc("sort")
        );

        PageData<CompanyVO> data= new PageConverter<Company, CompanyVO>() {
            @Override
            public CompanyVO converter(Company entity) {
                return CompanyConvertHelper.getCompanyVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<CompanyVO> list(CompanyListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<CompanyVO> data=new ArrayList<>();
        List<Company> list= companyService.list(new QueryWrapper<Company>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                .eq(StrUtil.isNotEmpty(params.getSysType()),"sys_type",params.getSysType())
                .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                .orderByDesc("sort")
        );
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(CompanyConvertHelper.getCompanyVO(item));
        });
        return data;
    }

//    @Override
//    public List<CompanyVO> listByTree(String id) {
//        List<CompanyVO> list = this.list(new CompanyListDTO());
//        if(CollUtil.isEmpty(list)){
//            return null;
//        }
//        return null;
//    }

    @Override
    public CompanyVO add(CompanyAddDTO params) {
        ValidatorUtils.validateEntity(params);
        Company entity=new Company();
        entity.setId(params.getId());
        entity.setName(params.getName());
        entity.setSysType(params.getSysType());
        entity.setLogo(params.getLogo());
        entity.setPid(params.getPid());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setType(params.getType());
        if(params.getPropertyJson()!=null){
            entity.setPropertyJson(JSON.toJSONString(params.getPropertyJson()));
        }
        entity.setRowsStatus(YN.YES.getCode());
        companyService.save(entity);
        return CompanyConvertHelper.getCompanyVO(entity);
    }

    @Override
    public CompanyVO update(CompanyUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        Company entity= companyService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setType(params.getType());
        if(StrUtil.isNotEmpty(params.getName())){
            entity.setName(params.getName());
        }
        entity.setLogo(params.getLogo());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());

        if(params.getPropertyJson()!=null){
            entity.setPropertyJson(JSON.toJSONString(params.getPropertyJson()));
        }
        companyService.updateById(entity);
        companyCacheService.removeById(entity.getId());
        return CompanyConvertHelper.getCompanyVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Company entity= companyService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        companyService.updateById(entity);
        companyCacheService.removeById(entity.getId());
    }

    @Override
    public CompanyVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Company entity= companyService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return CompanyConvertHelper.getCompanyVO(entity);
    }



    @Override
    public List<CompanyVO> tree(CompanyTreeDTO params) {
        List<CompanyVO> list = this.list(new CompanyListDTO());
        if(CollUtil.isEmpty(list)){
            return null;
        }
        if(StrUtil.isNotEmpty(params.getId())){
            return  TreeUtils.build(list,params.getId());
        }else {
            return TreeUtils.build(list);
        }
    }





}
