package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.framework.modules.biz.service.ProjectBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.ProjectInfo;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.ProjectVO;
import com.xxh.cloud.basic.framework.modules.biz.helper.ProjectConvertHelper;
import com.xxh.cloud.basic.framework.modules.base.service.ProjectInfoService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ProjectBizServiceImpl implements ProjectBizService {

    @Autowired
    private ProjectInfoService projectInfoService;


    @Override
    public PageData<ProjectVO> query(ProjectInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<ProjectInfo> page= projectInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<ProjectInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCode()),"code",params.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .orderByDesc("create_date")
        );
        PageData<ProjectVO> data= new PageConverter<ProjectInfo, ProjectVO>() {
            @Override
            public ProjectVO converter(ProjectInfo entity) {
                return ProjectConvertHelper.getProjectVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<ProjectVO> list(ProjectInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ProjectInfo> list= projectInfoService.list(
                new QueryWrapper<ProjectInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCode()),"code",params.getCode())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq("status",YN.YES.getCode())
                        .orderByDesc("create_date")
        );
        List<ProjectVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }

        list.forEach(item->{
            data.add(ProjectConvertHelper.getProjectVO(item));
        });

        return data;
    }

    @Override
    public void add(ProjectInfoAddDTO params) {
        ValidatorUtils.validateEntity(params);

        if(YN.YES.getCode().equals(params.getIsDef())){
            projectInfoService.update(new UpdateWrapper<ProjectInfo>()
                    .set("is_def",YN.NO.getCode())
                    .eq("is_def",YN.YES.getCode())
            );
        }

        ProjectInfo entity=new ProjectInfo();
        entity.setCode(params.getCode());
        entity.setName(params.getName());
        entity.setStatus(params.getStatus());
        entity.setDirector(params.getDirector());
        entity.setRemark(params.getRemark());
        if(params.getDataLinkConfig()!=null){
            entity.setDataLinkConfig(JSON.toJSONString(params.getDataLinkConfig()));
        }
        entity.setIsDef(params.getIsDef());
        entity.setRowsStatus(YN.YES.getCode());
        projectInfoService.save(entity);

    }

    @Override
    public void update(ProjectInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        ProjectInfo entity= projectInfoService.getById(params.getCode());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setDirector(params.getDirector());
        entity.setRemark(params.getRemark());
        if(params.getDataLinkConfig()!=null){
            entity.setDataLinkConfig(JSON.toJSONString(params.getDataLinkConfig()));
        }
        projectInfoService.updateById(entity);
    }

    @Override
    public void updateStatus(ProjectInfoEditStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        ProjectInfo entity= projectInfoService.getById(params.getCode());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(params.getStatus().equals(entity.getStatus())){
            return;
        }
        entity.setStatus(params.getStatus());
        projectInfoService.updateById(entity);
    }

    @Override
    public void updateDef(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        projectInfoService.update(new UpdateWrapper<ProjectInfo>()
                .set("is_def",YN.NO.getCode())
                .eq("is_def",YN.YES.getCode())
        );

        projectInfoService.update(new UpdateWrapper<ProjectInfo>()
                .set("is_def",YN.YES.getCode())
                .eq("code",id)
        );
        //todo:
    }

    @Override
    public void updateDataKey(ProjectInfoEditKeyDTO params) {
        ValidatorUtils.validateEntity(params);
        ProjectInfo entity= projectInfoService.getById(params.getCode());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //todo: 判断认证

        entity.setDataPrivateKey(params.getDataPrivateKey());
        entity.setDataPublicKey(params.getDataPublicKey());
        projectInfoService.updateById(entity);
    }

    @Override
    public ProjectVO getDataKeyById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ProjectInfo entity= projectInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //todo: 判断认证

        return ProjectConvertHelper.getProjectVO2DataKey(entity);
    }

    @Override
    public ProjectVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ProjectInfo entity= projectInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ProjectConvertHelper.getProjectVO2DS(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        ProjectInfo entity= projectInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        projectInfoService.updateById(entity);
    }
}
