package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.framework.modules.biz.entity.SiteMessageQueryBean;
import com.xxh.cloud.basic.framework.modules.biz.helper.SiteMessageConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.mapper.SiteMessageBizMapper;
import com.xxh.cloud.basic.framework.modules.biz.service.SiteMessageBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.SiteMessageInfo;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.SiteMessageUserInfo;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.SiteMessageAddDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.SiteMessageQueryDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.SiteMessageVO;
import com.xxh.cloud.basic.framework.modules.base.service.SiteMessageInfoService;
import com.xxh.cloud.basic.framework.modules.base.service.SiteMessageUserInfoService;
import com.xxh.cloud.basic.framework.modules.constant.SiteMessageUserStatus;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Log4j2
@Service
public class SiteMessageBizServiceImpl implements SiteMessageBizService {

    @Autowired
    private SiteMessageInfoService siteMessageInfoService;
    @Autowired
    private SiteMessageUserInfoService siteMessageUserInfoService;
    @Autowired
    private SiteMessageBizMapper siteMessageBizMapper;


    @Override
    public PageData<SiteMessageVO> query(SiteMessageQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<SiteMessageQueryBean> page= siteMessageBizMapper.queryUserSiteMessage(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<SiteMessageQueryBean>()
                        .eq("a.rows_status",YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getMessageId()),"a.id",params.getMessageId())
                        .like(StrUtil.isNotEmpty(params.getMessageTitle()),"a.title",params.getMessageTitle())
                        .eq(StrUtil.isNotEmpty(params.getMessageType()),"a.type",params.getMessageType())
                        .eq(StrUtil.isNotEmpty(params.getFromUserId()),"a.creator",params.getFromUserId())
                        .ge(StrUtil.isNotEmpty(params.getSendDateStart()),"a.create_date",params.getSendDateStart())
                        .le(StrUtil.isNotEmpty(params.getSendDateEnd()),"a.create_date",params.getSendDateEnd())

                        .ge(StrUtil.isNotEmpty(params.getReadDateStart()),"a.read_date",params.getReadDateStart())
                        .le(StrUtil.isNotEmpty(params.getReadDateEnd()),"a.read_date",params.getReadDateEnd())

                        .eq(StrUtil.isNotEmpty(params.getToUserId()),"b.to_user_id",params.getToUserId())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"b.status",params.getStatus())

                        .ne("b.status", SiteMessageUserStatus.Smu_Delete.getCode())
                        .orderByDesc("a.create_date")
                );


        //todo:user

        PageData<SiteMessageVO> data= new PageConverter<SiteMessageQueryBean, SiteMessageVO>() {
            @Override
            public SiteMessageVO converter(SiteMessageQueryBean entity) {
                return SiteMessageConvertHelper.getSiteMessageVO(entity);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public SiteMessageVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        SiteMessageUserInfo siteMessageUser = siteMessageUserInfoService.getById(id);
        if(siteMessageUser==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        SiteMessageInfo  siteMessage= siteMessageInfoService.getById(siteMessageUser.getSiteMessageId());
        if(siteMessage==null || YN.NO.getCode().equals(siteMessage.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        siteMessageUserInfoService.update(
                new UpdateWrapper<SiteMessageUserInfo>()
                        .set("status",SiteMessageUserStatus.Smu_Read.getCode())
                        .set("read_date",new Date())
                        .eq("id",id)
                        .eq("status",SiteMessageUserStatus.Smu_None.getCode())
        );

        //todo:user

        return SiteMessageConvertHelper.getSiteMessageVO(siteMessage,siteMessageUser);
    }

    @Override
    public void add(SiteMessageAddDTO params) {
        ValidatorUtils.validateEntity(params);
        SiteMessageInfo entity=SiteMessageConvertHelper.getSiteMessage_Add(params);
        List<SiteMessageUserInfo> siteMessageUserList=new ArrayList<>();
        params.getToUserIds().forEach(toUserId->{
            siteMessageUserList.add(SiteMessageConvertHelper.getSiteMessageUser_Add(entity.getId(),toUserId));
        });
        siteMessageInfoService.save(entity);
        siteMessageUserInfoService.saveBatch(siteMessageUserList);
    }

    @Override
    public void updateReadStatus(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        siteMessageUserInfoService.update(
                new UpdateWrapper<SiteMessageUserInfo>()
                        .set("status",SiteMessageUserStatus.Smu_Read.getCode())
                        .set("read_date",new Date())
                        .eq("id",id)
                        .eq("status",SiteMessageUserStatus.Smu_None.getCode())
        );

    }

    @Override
    public void updateReadStatusBatch(Set<String> ids) {
        if(CollUtil.isEmpty(ids)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        siteMessageUserInfoService.update(
                new UpdateWrapper<SiteMessageUserInfo>()
                        .set("status",SiteMessageUserStatus.Smu_Read.getCode())
                        .set("read_date",new Date())
                        .in("id",ids)
                        .eq("status",SiteMessageUserStatus.Smu_None.getCode())
        );
    }



    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        siteMessageUserInfoService.update(
                new UpdateWrapper<SiteMessageUserInfo>()
                        .set("status",SiteMessageUserStatus.Smu_Delete.getCode())
                        .eq("id",id)
        );
    }

    @Override
    public void delByIds(Set<String> ids) {
        if(CollUtil.isEmpty(ids)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        siteMessageUserInfoService.update(
                new UpdateWrapper<SiteMessageUserInfo>()
                        .set("status",SiteMessageUserStatus.Smu_Delete.getCode())
                        .in("id",ids)
        );
    }

}
