package com.xxh.cloud.basic.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.client.api.bean.dto.VisitVerificationSearchDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.VisitVerificationConfig;
import com.xxh.cloud.basic.framework.modules.base.service.VisitVerificationConfigService;
import com.xxh.cloud.basic.framework.modules.biz.helper.VisitVerificationConvertHelper;
import com.xxh.cloud.basic.framework.modules.cache.service.VisitVerificationCacheService;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class VisitVerificationCacheServiceImpl implements VisitVerificationCacheService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private VisitVerificationConfigService visitVerificationConfigService;

    @Override
    public void cacheVisitVerification() {
        List<VisitVerificationConfig> list = visitVerificationConfigService.list();
        if (CollUtil.isEmpty(list)) {
            return;
        }
        Map<String, List<VisitVerificationBO>> map = new HashMap<>();
        for (VisitVerificationConfig entity : list) {
            VisitVerificationBO bo = VisitVerificationConvertHelper.getVisitVerificationVO(entity);
            String cacheKey = VisitVerificationConvertHelper.getVisitVerificationCacheKey(bo.getVisitType(), bo.getVisitId());
            List<VisitVerificationBO> mapV;
            if (map.containsKey(cacheKey)) {
                mapV = map.get(cacheKey);
            } else {
                mapV = new ArrayList<>();
            }
            mapV.add(bo);
            map.put(cacheKey, mapV);
        }
        map.forEach((k, v) -> {
            redisUtils.set(k, v, 1800);
        });
    }

    @Override
    public Map<String, List<VisitVerificationBO>> getVisitVerification(VisitVerificationSearchDTO params) {
        if (StrUtil.isEmpty(params.getVisitType()) || CollUtil.isEmpty(params.getVisitIds())) {
            return new HashMap<>();
        }
        Map<String, List<VisitVerificationBO>> map = new HashMap<>();
        for (String visitId : params.getVisitIds()) {
            String cacheKey = VisitVerificationConvertHelper.getVisitVerificationCacheKey(params.getVisitType(), visitId);
            if (redisUtils.hasKey(cacheKey)) {
                map.put(visitId, (List<VisitVerificationBO>) redisUtils.get(cacheKey));
            }
        }
        return map;
    }

}
