package com.xxh.cloud.basic.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.client.api.CompanyClientService;
import com.xxh.cloud.basic.client.api.bean.bos.CompanyBO;
import com.xxh.cloud.basic.framework.modules.biz.service.CompanyBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.CompanyListDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.CompanyVO;
import com.xxh.cloud.basic.framework.modules.cache.service.CompanyCacheService;
import com.xxh.cloud.basic.framework.modules.client.helper.BasicClientConvertHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class CompanyClientLocalService implements CompanyClientService {


    @Autowired
    private CompanyCacheService companyCacheService;
    @Autowired
    private CompanyBizService companyBizService;

    @Override
    public CompanyBO getCompanyById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        CompanyVO company = companyCacheService.getById(id);
        return BasicClientConvertHelper.getCompanyBO(company);
    }

    @Override
    public List<CompanyBO> getCompanyListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<CompanyVO> list = companyCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(BasicClientConvertHelper::getCompanyBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, CompanyBO> getCompanyMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<CompanyVO> list = companyCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(CompanyVO::getId, BasicClientConvertHelper::getCompanyBO)));
    }

    @Override
    public List<CompanyBO> getCompanyList() {
        List<CompanyBO> data=new ArrayList<>();

        CompanyListDTO params=new CompanyListDTO();
        List<CompanyVO> list= companyBizService.list(params);
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(BasicClientConvertHelper.getCompanyBO(item));
        });
        return data;
    }


}
