package com.xxh.cloud.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.CompanyVO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictDataVO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictTypeTreeVO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.DictTypeVO;
import com.xxh.cloud.basic.framework.modules.base.service.DictDataService;
import com.xxh.cloud.basic.framework.modules.base.service.DictTypeService;
import com.xxh.cloud.basic.framework.modules.constant.CompanySysType;
import com.xxh.cloud.basic.framework.modules.constant.DictTypeCategory;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:basic:dict")
@RestController
@RequestMapping("/sys/basic/dict")
@Api(tags="运营端_管理,字典管理")
public class DictManagerController {


    @Autowired
    private DictTypeService dictTypeService;
    @Autowired
    private DictDataService dictDataService;


    @PostMapping("queryDictType")
    @ApiOperation("查询字典分类")
    @SaCheckPermission("sys:basic:dict:queryDictType")
    public Result<PageData<DictTypeVO>> queryDictType(@RequestBody DictTypeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DictTypeVO> data= dictTypeService.query(params);
        return new Result<PageData<DictTypeVO>>().ok(data);
    }

    @PostMapping("listDictType")
    @ApiOperation("字典分类列表")
    @SaCheckPermission("sys:basic:dict:listDictType")
    public Result<List<DictTypeVO>> listDictType(@RequestBody DictTypeListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DictTypeVO> data= dictTypeService.list(params);
        return new Result<List<DictTypeVO>>().ok(data);
    }


    @PostMapping("listDictTree")
    @ApiOperation("字典分类树")
    @SaCheckPermission("sys:basic:dict:listDictTree")
    public Result<List<DictTypeTreeVO>> listDictTree() {
        List<DictTypeTreeVO>  data= dictTypeService.tree();
        return new Result<List<DictTypeTreeVO>>().ok(data);
    }

    @PostMapping("addDictType")
    @ApiOperation("新增字典分类")
    @SaCheckPermission("sys:basic:dict:addDictType")
    public Result<DictTypeVO> addDictType(@RequestBody DictTypeAddDTO params) {
        if(StrUtil.isEmpty(params.getId())){
            params.setId(IdUtil.getSnowflake(2,2).nextIdStr());
        }
        if(StrUtil.isEmpty(params.getType())){
            params.setType(DictTypeCategory.Biz_Dict.getCode());
        }
        ValidatorUtils.validateEntity(params);
        DictTypeVO data= dictTypeService.add(params);
        return new Result<DictTypeVO>().ok(data);
    }


    @PostMapping("updateDictType")
    @ApiOperation("修改字典分类")
    @SaCheckPermission("sys:basic:dict:updateDictType")
    public Result<DictTypeVO> updateDictType(@RequestBody DictTypeUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DictTypeVO data= dictTypeService.update(params);
        return new Result<DictTypeVO>().ok(data);
    }


    @PostMapping("infoDictType")
    @ApiOperation("获取字典分类")
    @SaCheckPermission("sys:basic:dict:infoDictType")
    public Result<DictTypeVO> infoDictType(@RequestBody DictTypeInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        DictTypeVO data=dictTypeService.infoById(params.getId());
        return new Result<DictTypeVO>().ok(data);
    }


    @PostMapping("delDictType")
    @ApiOperation("删除字典分类")
    @SaCheckPermission("sys:basic:dict:delDictType")
    public Result<String> delDictType(@RequestBody DictTypeInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        dictTypeService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


    //===============================字典数据=============================================

    @PostMapping("listDictInfo")
    @ApiOperation("字典列表")
    @SaCheckPermission("sys:dict:listDictInfo")
    public Result<List<DictDataVO>> listDictInfo(@RequestBody DictDataListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DictDataVO> data= dictDataService.list(params);
        return new Result<List<DictDataVO>>().ok(data);
    }


    @PostMapping("addDictInfo")
    @ApiOperation("新增字典")
    @SaCheckPermission("sys:dict:addDictInfo")
    public Result<DictDataVO> addDictInfo(@RequestBody DictDataAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DictDataVO data= dictDataService.add(params);
        return new Result<DictDataVO>().ok(data);
    }


    @PostMapping("updateDictInfo")
    @ApiOperation("修改字典")
    @SaCheckPermission("sys:dict:updateDictInfo")
    public Result<DictDataVO> updateDictInfo(@RequestBody DictDataUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DictDataVO data= dictDataService.update(params);
        return new Result<DictDataVO>().ok(data);
    }


    @PostMapping("add2updateBatchDictInfo")
    @ApiOperation("批量新增修改字典")
    @SaCheckPermission("sys:dict:add2updateBatchDictInfo")
    public Result<String> add2updateBatchDictInfo(@RequestBody DictDataAdd2UpdateBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        dictDataService.save2updateBatch(params);
        return new Result<String>().ok("ok");
    }

    @PostMapping("getDictInfo")
    @ApiOperation("获取字典明细")
    @SaCheckPermission("sys:dict:getDictInfo")
    public Result<DictDataVO> getDictInfo(@RequestBody DictDataInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        DictDataVO data= dictDataService.infoById(params.getId());
        return new Result<DictDataVO>().ok(data);
    }

    @PostMapping("delDictInfo")
    @ApiOperation("删除字典")
    @SaCheckPermission("sys:dict:delDictInfo")
    public Result<String> delDictInfo(@RequestBody DictDataInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        dictDataService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


}
