package com.xxh.cloud.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.ParamsVO;
import com.xxh.cloud.basic.framework.modules.base.service.ParamsService;
import com.xxh.cloud.basic.framework.modules.biz.service.ParamsBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.param.constant.ParamTypeEnum;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:basic:params")
@RestController
@RequestMapping("/sys/basic/params")
@Api(tags="运营端_管理,参数管理")
public class ParamsManagerController {


    @Autowired
    private ParamsBizService paramsBizService;


    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:basic:params:query")
    public Result<PageData<ParamsVO>> query(@RequestBody ParamsQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ParamsVO> data= paramsBizService.query(params);
        return new Result<PageData<ParamsVO>>().ok(data);
    }


    @PostMapping("list")
    @ApiOperation("列表")
    @SaCheckPermission("sys:basic:params:list")
    public Result<List<ParamsVO>> list(@RequestBody ParamsListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ParamsVO> data= paramsBizService.list(params);
        return new Result<List<ParamsVO>>().ok(data);
    }



    @PostMapping("listMetadataValueConfig")
    @ApiOperation("(标准数据)列表")
    @SaCheckPermission("sys:basic:params:listMetadataValueConfig")
    public Result<List<ParamsVO>> listMetadataValueConfig(@RequestBody ParamsMetaListDTO params) {
        ValidatorUtils.validateEntity(params);
        ParamsListDTO listParam=new ParamsListDTO();
        listParam.setParamType(ParamTypeEnum.BizParamList.getCode());
        params.getDataTypeList().forEach(item->{
            listParam.addParamValueType(StrUtil.format(ParamValTypeEnum.MetadataValue_Config.getCode(),item));
        });
        List<ParamsVO> data= paramsBizService.list(listParam);
        return new Result<List<ParamsVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:basic:params:add")
    public Result<ParamsVO> add(@RequestBody ParamsAddDTO params) {
        ValidatorUtils.validateEntity(params);
        ParamsVO data= paramsBizService.add(params);
        return new Result<ParamsVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:basic:params:update")
    public Result<ParamsVO> update(@RequestBody ParamsUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        ParamsVO data= paramsBizService.update(params);
        return new Result<ParamsVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:basic:params:info")
    public Result<ParamsVO> info(@RequestBody ParamsInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        ParamsVO data=paramsBizService.infoById(params.getId());
        return new Result<ParamsVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:basic:params:del")
    public Result<String> del(@RequestBody ParamsInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        paramsBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }





}
