package com.xxh.cloud.customer.framework.modules.base.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerUserInfo;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.CustomerUserVO;
import com.xxh.cloud.customer.framework.modules.biz.helper.CustomerUserHelper;
import com.xxh.cloud.customer.framework.modules.base.mapper.CustomerUserInfoMapper;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 客户用户信息表 服务实现类
 */
@Service
public class CustomerUserInfoServiceImpl extends ServiceImpl<CustomerUserInfoMapper, CustomerUserInfo> implements CustomerUserInfoService {


    @Autowired
    private RedisUtils redisUtils;

    @Override
    public PageData<CustomerUserVO> query(UserCustomerQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<CustomerUserInfo> page=this.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<CustomerUserInfo>()
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())

                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .eq(StrUtil.isNotEmpty(params.getSourceOne()),"source_one",params.getSourceOne())
                        .eq(StrUtil.isNotEmpty(params.getSourceTwo()),"source_two",params.getSourceTwo())
                        .eq(StrUtil.isNotEmpty(params.getSourceThree()),"source_three",params.getSourceThree())
                        .eq(StrUtil.isNotEmpty(params.getShareCode()),"share_code",params.getShareCode())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .or(StrUtil.isNotEmpty(params.getName()),
                                i->i.like("user_name",params.getName())
                                        .or().like("real_name",params.getName())
                                        .or().eq("mobile",params.getName())
                                        .or().eq("email",params.getName())
                        )
                        .orderByDesc("create_date")
        );
        PageData<CustomerUserVO> data= new PageConverter<CustomerUserInfo, CustomerUserVO>() {
            @Override
            public CustomerUserVO converter(CustomerUserInfo user) {
                return CustomerUserHelper.getCustomerUserVO(user,params.isDesensitized());
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<CustomerUserVO> list(UserCustomerListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<CustomerUserInfo> list=this.list(new QueryWrapper<CustomerUserInfo>()
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
        );
        List<CustomerUserVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(CustomerUserHelper.getCustomerUserVO(item,params.isDesensitized()));
            });
        }

        return data;
    }


    @Override
    public CustomerUserVO update(UserCustomerUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        CustomerUserInfo entity=this.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setNickName(params.getNickName());
        entity.setRealName(params.getRealName());
        entity.setHeadUrl(params.getHeadUrl());
        if (MapUtil.isNotEmpty(params.getCustomerExtend())) {
            entity.setCustomerExtend(JSONObject.toJSONString(params.getCustomerExtend()));
        }
        this.updateById(entity);

        redisUtils.delete(CacheCustomerKeyHelper.getCustomerUserCacheKey(entity.getId()));
        return CustomerUserHelper.getCustomerUserVO(entity,params.isDesensitized());
    }

    @Override
    public CustomerUserVO updateStatus(UserCustomerUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        if(StrUtil.isEmpty(params.getId())){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerUserInfo entity=this.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

//        if(params.get){
//
//        }
        redisUtils.delete(CacheCustomerKeyHelper.getCustomerUserCacheKey(entity.getId()));
        return null;
    }

    @Override
    public CustomerUserVO updatePassword(UserCustomerUpdatePasswordDTO params) {
        ValidatorUtils.validateEntity(params);
        return null;
    }

    @Override
    public CustomerUserVO updateMobile(UserCustomerUpdateMobileDTO params) {
        ValidatorUtils.validateEntity(params);
        return null;
    }

    @Override
    public CustomerUserVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerUserInfo entity=this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return CustomerUserHelper.getCustomerUserVO(entity,false);
    }

    @Override
    public CustomerUserVO infoById(String id, boolean isDesensitized) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerUserInfo entity=this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return CustomerUserHelper.getCustomerUserVO(entity,isDesensitized);
    }

    @Override
    public CustomerUserVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
           return null;
        }
        CustomerUserInfo entity=this.getById(id);
        if(entity==null){
            return null;
        }
        return CustomerUserHelper.getCustomerUserVO(entity,false);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerUserInfo entity=this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //todo:

        redisUtils.delete(CacheCustomerKeyHelper.getCustomerUserCacheKey(entity.getId()));
    }


}
