package com.xxh.cloud.customer.framework.modules.biz.mapper;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyMemberInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerUserInfo;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisPatientModel;

public interface CustomerDwdPatientBizMapper {

    /**
     * 患者门诊id换取患者主id
     */
    @Select({"<script>",
                " SELECT main_person_id mainPersonId, patient_id patientId ",
                " FROM dwd_patient_info ",
                " WHERE patient_id in ",
                " <foreach collection='patientIds' open='(' close=')' separator=',' item='patientId'> ",
                "   #{patientId} ",
                " </foreach> ",
            " </script> "})
    List<Map<String, String>> getMainPersonIdsByPatientIds(@Param("patientIds") List<String> patientIds);

    // 保存互联网医院家属数据
    @Insert({
        "<script>",
        "INSERT INTO cust_his_customer_user_info (id, his_patient_id, in_patient_no, name, paper_no, sex, birthday, native_place, link_phone, link_name, link_relation_name, is_default, async_flag, open_id, unionid, create_time, update_time, creator, create_date, updater, update_date) VALUES ",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "(#{item.id}, #{item.hisPatientId}, #{item.inPatientNo}, #{item.name}, #{item.paperNo}, #{item.sex}, #{item.birthday}, #{item.nativePlace}, #{item.linkPhone}, #{item.linkName}, #{item.linkRelationName}, #{item.isDefault}, '1', #{item.openId}, #{item.unionid}, #{item.createDate}, #{item.updateDate}, 'sync', NOW(), 'sync', NOW() )",
        "</foreach>",
        "</script>"
    })
    Long syncHisPatient(@Param("dataList") List<HisPatientModel> dataList);

    // 保存创建的互联网医院家属用户数据
    @Insert({
        "<script>",
        "INSERT INTO cust_customer_user_info (id, user_name, pass_word, share_code, status, creator, create_date, updater, update_date) VALUES ",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "(#{item.id}, #{item.userName}, #{item.passWord}, #{item.shareCode}, #{item.status}, 'sync', NOW(), 'sync', NOW() )",
        "</foreach>",
        "</script>"
    })
    Long saveHisPatientUser(@Param("dataList") List<CustomerUserInfo> dataList);

    // 保存创建的互联网医院家属第三方用户数据
    @Insert({
        "<script>",
        "INSERT INTO cust_customer_third_user_info (id, params_id, open_id, union_id, user_id, creator, create_date, updater, update_date) VALUES ",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "(#{item.id}, #{item.paramsId}, #{item.openId}, #{item.unionId}, #{item.userId}, 'sync', NOW(), 'sync', NOW() )",
        "</foreach>",
        "</script>"
    })
    Long saveHisPatientThirdUser(@Param("dataList") List<CustomerThirdUserInfo> dataList);

    // 保存创建的互联网医院家属第三方用户数据
    @Insert({
        "<script>",
        "UPDATE cust_customer_third_user_info SET update_date = NOW(), open_id = CASE union_id ",
        "<foreach collection='dataList' item='item' index='index' separator=' '>",
        "  WHEN #{item.unionId} THEN #{item.openId} ",
        "</foreach>",
        " END ",
        " WHERE union_id IN (",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "  #{item.unionId} ",
        "</foreach>",
        " ) ",
        "</script>"
    })
    Long updateHisPatientThirdUser(@Param("dataList") List<CustomerThirdUserInfo> dataList);

    // 保存创建的互联网医院家属关系数据
    @Insert({
        "<script>",
        "INSERT INTO cust_customer_family_member_info (id, user_id, member_relation, name, mobile, card_type, card_no, status, rows_status, creator, create_date, updater, update_date, gender, patient_id, pat_his_no, relation_name, relation_type, main_person_id, create_time, update_time, relation_flag, is_auth) VALUES ",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "(#{item.id}, #{item.userId}, #{item.memberRelation}, #{item.name}, #{item.mobile}, #{item.cardType}, #{item.cardNo}, #{item.status}, 'Y', 'sync', NOW(), 'sync', NOW(), #{item.gender}, #{item.patientId}, #{item.patHisNo}, #{item.relationName}, #{item.relationType}, #{item.mainPersonId}, #{item.createTime}, #{item.updateTime}, #{item.relationFlag}, #{item.isAuth} )",
        "</foreach>",
        "</script>"
    })
    Long saveHisPatientFamilyUser(@Param("dataList") List<CustomerFamilyMemberInfo> dataList);

    // 保存创建的互联网医院家属关系数据
    @Insert({
        "<script>",
        "UPDATE cust_customer_family_member_info SET update_date = NOW(), rows_status = CASE id ",
        "<foreach collection='dataList' item='item' index='index' separator=' '>",
        "  WHEN #{item.id} THEN #{item.rowsStatus} ",
        "</foreach>",
        " END ",
        " WHERE id IN (",
        "<foreach collection='dataList' item='item' index='index' separator=','>",
        "  #{item.id} ",
        "</foreach>",
        " ) ",
        "</script>"
    })
    Long updateHisPatientFamilyUser(@Param("dataList") List<CustomerFamilyMemberInfo> dataList);

    // 设置默认本人关系
    @Update("UPDATE cust_customer_family_member_info SET update_date = NOW(), member_relation = '本人', relation_name = '本人', relation_type = '1', is_auth = 'Y' WHERE id IN ( " +
            "    SELECT id FROM ( " +
            "       SELECT a.id " +
            "       FROM cust_customer_family_member_info a " +
            "       INNER JOIN ( " +
            "           SELECT user_id FROM cust_customer_family_member_info WHERE rows_status = 'Y' GROUP BY user_id HAVING COUNT(1) = 1 " +
            "       ) b ON a.user_id = b.user_id " +
            "       WHERE a.is_auth = 'N' " +
            "    ) a " +
            ")")
    Long setSelfRelationTask();

}
