package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerAddrInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerAddrInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.AddressVO;
import com.xxh.cloud.customer.framework.modules.biz.helper.CustomerAddrHelper;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerAddrBizService;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Service
public class CustomerAddrBizServiceImpl implements CustomerAddrBizService {


    @Autowired
    private CustomerAddrInfoService customerAddrInfoService;


    @Autowired
    private RedisUtils redisUtils;

    @Override
    public PageData<AddressVO> query(AddressQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<CustomerAddrInfo> page=customerAddrInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<CustomerAddrInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getUserId()),"user_id",params.getUserId())
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .orderByDesc("create_date")
        );

        PageData<AddressVO> data= new PageConverter<CustomerAddrInfo, AddressVO>() {
            @Override
            public AddressVO converter(CustomerAddrInfo entity) {
                return CustomerAddrHelper.getAddressVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<AddressVO> list(AddressListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<CustomerAddrInfo> list=customerAddrInfoService.list(new QueryWrapper<CustomerAddrInfo>()
                .eq("rows_status",YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getUserId()),"user_id",params.getUserId())
                .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
        );
        List<AddressVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(CustomerAddrHelper.getAddressVO(item));
            });
        }
        return data;
    }

    @Override
    public List<AddressVO> listByUser(String userId) {
        if(StrUtil.isEmpty(userId)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        List<CustomerAddrInfo> list= customerAddrInfoService.list(new QueryWrapper<CustomerAddrInfo>()
                .eq("user_id",userId)
                .eq("rows_status",YN.YES.getCode())
                .orderByDesc("create_date")
        );
        List<AddressVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(CustomerAddrHelper.getAddressVO(item));
            });
        }
        return data;
    }

    @Override
    public AddressVO add(AddressAddDTO params) {
        ValidatorUtils.validateEntity(params);
        CustomerAddrInfo entity= CustomerAddrHelper.getCustomerAddrInfo_Add(params);
        if(YN.YES.getCode().equals(entity.getIsDef())){
            customerAddrInfoService.update(new UpdateWrapper<CustomerAddrInfo>()
                    .set("is_def",YN.NO.getCode())
                    .eq("user_id",params.getUserId())
                    .eq("is_def",YN.YES.getCode())
                    .eq("rows_status",YN.YES.getCode())
            );
        }
        customerAddrInfoService.save(entity);
        return CustomerAddrHelper.getAddressVO(entity);
    }

    @Override
    public AddressVO update(AddressUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        CustomerAddrInfo entity=customerAddrInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        CustomerAddrHelper.getCustomerAddrInfo_Update(entity,params);
        customerAddrInfoService.updateById(entity);
        redisUtils.delete(CacheCustomerKeyHelper.getCustomerAddrCacheKey(entity.getId()));
        return CustomerAddrHelper.getAddressVO(entity);
    }

    @Override
    public void updateDef(AddressUpdateDefDTO params) {
        ValidatorUtils.validateEntity(params);
        CustomerAddrInfo entity=customerAddrInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        if(YN.YES.getCode().equals(entity.getIsDef())){
            customerAddrInfoService.update(new UpdateWrapper<CustomerAddrInfo>()
                    .set("is_def",YN.NO.getCode())
                    .eq("user_id",entity.getUserId())
                    .eq("is_def",YN.YES.getCode())
                    .eq("rows_status",YN.YES.getCode())
            );
        }

        entity.setIsDef(params.getIsDef());
        customerAddrInfoService.updateById(entity);
    }

    @Override
    public AddressVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerAddrInfo entity=customerAddrInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return CustomerAddrHelper.getAddressVO(entity);
    }

    @Override
    public AddressVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        CustomerAddrInfo entity=customerAddrInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        return CustomerAddrHelper.getAddressVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerAddrInfo entity=customerAddrInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setIsDef(YN.NO.getCode());
        entity.setRowsStatus(YN.NO.getCode());
        customerAddrInfoService.updateById(entity);
        redisUtils.delete(CacheCustomerKeyHelper.getCustomerAddrCacheKey(entity.getId()));
    }

}
