package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyMemberInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerFamilyMemberInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.MemberFamilyAuthDTO;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberAuthService;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.customer.framework.modules.constant.RelationTypeEnum;
import com.xxh.cloud.customer.framework.modules.constant.XXCustUserErrorEnum;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.healthy.serve.pushServe.client.api.PatientPushServeClientService;
import com.xxh.cloud.healthy.serve.pushServe.client.api.bean.dto.PatientScheduleCreateClientDTO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Log4j2
@Service
public class CustomerFamilyMemberAuthServiceImpl implements CustomerFamilyMemberAuthService {

    @Autowired
    private CustomerFamilyMemberInfoService customerFamilyMemberInfoService;
//    @Autowired
//    private PatientPushServeClientService patientPushServeClientService;
    @Autowired
    private RedisUtils redisUtils;


    private boolean checkAuthCode(MemberFamilyAuthDTO params){
        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberAuthKey(params.getFamilyMemberId());
        if (redisUtils.hasKey(redisKey) && redisUtils.get(redisKey).equals(params.getAuthCode())) {
            redisUtils.delete(redisKey);
            return true;
        }else {
            return false;
        }
    }
    @Override
    public void bind(MemberFamilyAuthDTO params){
        CustomerFamilyMemberInfo entity = customerFamilyMemberInfoService.getById(params.getFamilyMemberId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 校验本人唯一
        checkRelationSelf(entity, params.getRelationType());
        entity.setIsAuth(params.getIsAuth());
        entity.setAuthDate(new Date());
        // 更新用户关系
        if (StrUtil.isNotEmpty(params.getRelationType())) {
            entity.setRelationType(params.getRelationType());
            entity.setMemberRelation(RelationTypeEnum.getEnabledEnum(params.getRelationType()).getName());
            entity.setRelationName(RelationTypeEnum.getEnabledEnum(params.getRelationType()).getName());
        }
        // 保存数据来源
        entity.setSourceOne(params.getSourceOne());
        entity.setSourceTwo(params.getSourceTwo());
        entity.setSourceThree(params.getSourceThree());
        if (RelationTypeEnum.self.getCode().equals(entity.getRelationType())) {
            entity.setRelationFlag("");
        }
        customerFamilyMemberInfoService.updateById(entity);
        
        // 生成日程的client
        try {
            PatientScheduleCreateClientDTO createClient = new PatientScheduleCreateClientDTO();
            createClient.setPatientId(entity.getPatHisNo());
            createClient.setUserId(entity.getUserId());
//            patientPushServeClientService.createSchedule(createClient);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("绑定患者时生成日程失败", e.getMessage());
        }
    }


    @Override
    public void auth(MemberFamilyAuthDTO params) {
        boolean flag= this.checkAuthCode(params);
        if (flag) {
           this.bind(params);
        } else {
            throw new BusinessException(XXCustUserErrorEnum.FAMILY_AUTH_CODE_ERR);
        }
    }


    // 校验本人关系 PID只能有一个本人
    private void checkRelationSelf(CustomerFamilyMemberInfo patient, String relationType){
        if (relationType.equals(RelationTypeEnum.self.getCode())) {
            // 校验用户的本人关系唯一
            long userSelfCount = customerFamilyMemberInfoService.count(new QueryWrapper<CustomerFamilyMemberInfo>()
                    .eq("user_id", patient.getUserId())
                    .ne("id", patient.getId())
                    .eq("relation_type", RelationTypeEnum.self.getCode())
                    .eq("rows_status", YN.YES.getCode()));
            if (userSelfCount > 0) {
                throw new BusinessException(XXCustUserErrorEnum.RELATION_SELF_ONLY);
            }

            // 校验家属的本人关系唯一
            long patSelfCount = customerFamilyMemberInfoService.count(new QueryWrapper<CustomerFamilyMemberInfo>()
                    .eq("patient_id", patient.getPatientId())
                    .ne("id", patient.getId())
                    .eq("relation_type", RelationTypeEnum.self.getCode())
                    .eq("rows_status", YN.YES.getCode()));
            if (patSelfCount > 0) {
                throw new BusinessException(XXCustUserErrorEnum.RELATION_SELF_ONLY);
            }
        }
    }
}
