package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.validation.ValidationUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerLogThird;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerLogThirdService;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.LogThirdDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.ThirdUserUpdateDTO;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerThirdUserBizService;
import com.xxh.cloud.customer.framework.modules.constant.LogThirdTypeEnum;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class CustomerThirdUserBizServiceImpl implements CustomerThirdUserBizService {

    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private CustomerLogThirdService customerLogThirdService;

    @Override
    public void updateThirdUser(ThirdUserUpdateDTO params) {
        ValidationUtil.validate(params);
        CustomerThirdUserInfo entity;
        if (StrUtil.isNotEmpty(params.getThirdUserId())) {
            entity = customerThirdUserInfoService.getById(params.getThirdUserId());
        } else if (StrUtil.isNotEmpty(params.getOpenId())) {
            entity = customerThirdUserInfoService.getOne(new QueryWrapper<CustomerThirdUserInfo>()
                            .eq("open_id", params.getOpenId()));
        } else {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if (entity == null) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        String opBefore = JSONObject.toJSONString(entity);
        if (StrUtil.isNotEmpty(params.getThirdUserStatus()) &&
                params.getThirdUserStatus().equals(entity.getThirdUserStatus())) {
            // 不需要更新
            return;
        }
        entity.setThirdUserStatus(params.getThirdUserStatus());
        customerThirdUserInfoService.updateById(entity);
        String opAfter = JSONObject.toJSONString(entity);
        // 记录更新日志
        addLogThird(new LogThirdDTO(LogThirdTypeEnum.THIRD_USER.getCode(), entity.getId(), opBefore, opAfter));
    }

    @Override
    public void addLogThird(LogThirdDTO params) {
        CustomerLogThird entity = new CustomerLogThird();
        entity.setId(IdUtil.getSnowflake(6,1).nextIdStr());
        entity.setOpType(params.getOpType());
        entity.setOpId(params.getOpId());
        entity.setOpBefore(params.getOpBefore());
        entity.setOpAfter(params.getOpAfter());
        customerLogThirdService.save(entity);

    }
}
