package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyMemberInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerFamilyMemberInfoService;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.helper.CustomerFamilyHelper;
import com.xxh.cloud.customer.framework.modules.biz.mapper.CustomerDwdPatientBizMapper;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerDwdPatientBizService;
import com.xxh.cloud.customer.framework.modules.biz.service.HisCustomerUserBizService;
import com.xxh.cloud.customer.framework.modules.login.helper.CustomerUserLoginHelper;
import com.xxh.cloud.customer.framework.modules.register.helper.CustomerUserRegisterHelper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.third.framework.modules.thirdHis.HisWebApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisPatientDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisPatientListModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisPatientModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 互联网医院客户用户信息
 */
@Service
public class HisCustomerUserBizServiceImpl implements HisCustomerUserBizService {

    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private CustomerDwdPatientBizMapper customerDwdPatientBizMapper;
    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private CustomerFamilyMemberInfoService customerFamilyMemberInfoService;
    @Autowired
    private CustomerDwdPatientBizService dwdPatientBizService;

    @Override
    public Boolean syncHisPatient(HisPatientDTO params) {
        HealthyFrontendConfig healthyFrontendConfig = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // 查询互联网医院患者信息
        RespBaseThird<HisPatientListModel> hisResp = HisWebApiHelper.getPatientByTimestamp(healthyFrontendConfig, params);
        if (null == hisResp.getData() || CollUtil.isEmpty(hisResp.getData().getPatients())) {
            return false;
        }

        List<HisPatientModel> patientFamilyList = hisResp.getData().getPatients(); // 数据有重复需要去重

        Map<String, String> unionidOpenidMap = new HashMap<>(); // key:unionid value:openid
        Map<String, List<CustomerFamilyMemberInfo>> hisPatientMap = new HashMap<>(); // key为unionid
        patientFamilyList.forEach(i -> {
            i.setId(StrUtil.format("FM{}", IdUtil.getSnowflake(4,2).nextIdStr()));

            if (StrUtil.isNotBlank(i.getUnionid())) {
                if (!unionidOpenidMap.containsKey(i.getUnionid()) || StrUtil.isBlank(unionidOpenidMap.get(i.getUnionid()))) {
                    unionidOpenidMap.put(i.getUnionid(), i.getOpenId()); // 初始openid可为null，但后续有值后不会再更新为null
                }

                List<CustomerFamilyMemberInfo> l = hisPatientMap.get(i.getUnionid());
                if (null == l) {
                    l = new ArrayList<>();
                    hisPatientMap.put(i.getUnionid(), l);
                }

                CustomerFamilyMemberInfo familyAddDTO = new CustomerFamilyMemberInfo();
                
                familyAddDTO.setId(CustomerFamilyHelper.getFamilyMemberId());
                // familyAddDTO.setUserId(data.getId());
                familyAddDTO.setIsAuth(YN.NO.getCode());
                familyAddDTO.setRelationName(i.getRelation());
                familyAddDTO.setRelationType(relationType(i.getRelationId()));
                if (StrUtil.isNotBlank(familyAddDTO.getRelationType())) { // 有关系则设为授权
                    familyAddDTO.setIsAuth(YN.YES.getCode());
                }
                familyAddDTO.setName(i.getName());
                familyAddDTO.setMobile(i.getLinkPhone());
                familyAddDTO.setCardType("1"); // 1:二代身份证 2:港澳居民身份证 3:台湾居民身份证 4:护照
                familyAddDTO.setCardNo(i.getPaperNo());
                familyAddDTO.setStatus(YN.YES.getCode());
                familyAddDTO.setRowsStatus(YN.YES.getCode());
                familyAddDTO.setBirthday(i.getBirthday());
                familyAddDTO.setPatientId(i.getHisPatientId());
                familyAddDTO.setGender("女".equals(i.getSex()) ? "F" : "M");
                familyAddDTO.setPatHisNo(i.getHisPatientId());
                familyAddDTO.setCreateTime(i.getCreateDate());
                familyAddDTO.setUpdateTime(i.getUpdateDate());
                familyAddDTO.setRelationFlag("1");

                if ("0".equals(i.getEnable())) {// 数据解绑
                    familyAddDTO.setRowsStatus(YN.NO.getCode());
                }

                l.add(familyAddDTO);
            }
        });
        // 保存互联网医院家属信息
        customerDwdPatientBizMapper.syncHisPatient(patientFamilyList);

        // 生成患者账号及患者第三方信息
        if (CollUtil.isNotEmpty(unionidOpenidMap)) {
            Map<String, String> unionidUserIdMap = new HashMap<>(); // k:unionid v:userid
            Set<String> userIds = new HashSet<>();
            
            Map<String, CustomerThirdUserInfo> ctuserMap = new HashMap<>(); // k:unionid
            List<CustomerThirdUserInfo> ctuserList = customerThirdUserInfoService.list(new QueryWrapper<CustomerThirdUserInfo>()
                .in("union_id", unionidOpenidMap.keySet())
            );
            if (CollUtil.isNotEmpty(ctuserList)) {
                ctuserList.forEach(i -> ctuserMap.put(i.getUnionId(), i));
            }

            List<CustomerUserInfo> custUserSaveList = new ArrayList<>(); // 新增用户数据
            List<CustomerThirdUserInfo> thirdUserSaveList = new ArrayList<>(); // 新增第三方数据
            List<CustomerThirdUserInfo> thirdUserUpdateList = new ArrayList<>(); // 更新第三方数据(主要更新openid)

            unionidOpenidMap.forEach((k, v) -> {
                if (!ctuserMap.keySet().contains(k)) {
                    CustomerThirdUserInfo thirdUser = new CustomerThirdUserInfo();
                    thirdUser.setId(CustomerUserLoginHelper.getCustomerThirdUserId());
                    thirdUser.setParamsId("WX_MP_3");
                    thirdUser.setUnionId(k);
                    thirdUser.setOpenId(v);

                    CustomerUserInfo entity = CustomerUserRegisterHelper.getCustomerUserInfo(thirdUser);
                    custUserSaveList.add(entity);
                    
                    thirdUser.setUserId(entity.getId());
                    thirdUserSaveList.add(thirdUser);

                    unionidUserIdMap.put(k, entity.getId());
                    userIds.add(entity.getId());
                } else {
                    CustomerThirdUserInfo thirdUser = ctuserMap.get(k);
                    if (StrUtil.isNotBlank(v)) {
                        thirdUser.setOpenId(v);
                        thirdUserUpdateList.add(thirdUser);
                    }

                    unionidUserIdMap.put(k, ctuserMap.get(k).getUserId());
                    userIds.add(ctuserMap.get(k).getUserId());
                }
            });

            Map<String, CustomerFamilyMemberInfo> familyMap = new HashMap<>(); // k: userid-patHisNo 需要新增的数据
            if (CollUtil.isNotEmpty(unionidUserIdMap)) {
                unionidUserIdMap.forEach((k, v) -> {
                    List<CustomerFamilyMemberInfo> l = hisPatientMap.get(k);
                    if (CollUtil.isNotEmpty(l)) {
                        l.forEach(i -> {
                            i.setUserId(v);
                            familyMap.put(v + "-" + i.getPatHisNo(), i); // 也去重
                        });
                    }
                });
            }

            // 查询存在的家属
            Map<String, CustomerFamilyMemberInfo> familyExistMap = new HashMap<>(); // k: userid-patHisNo 已经存在的家属信息
            List<CustomerFamilyMemberInfo> familyExistList = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
                // .eq("rows_status", YN.YES.getCode())  // 本状态为解绑/绑定状态，不能限制
                .in("user_id", userIds)
            );
            if (CollUtil.isNotEmpty(familyExistList)) {
                for (CustomerFamilyMemberInfo i : familyExistList) {
                    familyExistMap.put(i.getUserId() + "-" + i.getPatHisNo(), i);
                }
            }

            // 从需要新增的数据中筛选出已经存在的数据
            Set<CustomerFamilyMemberInfo> updateList = new HashSet<>(); // 更新绑定关系，根据同步的绑定状态不一致来确定：1.解绑的绑定，2.绑定的解绑
            if (CollUtil.isNotEmpty(familyExistMap)) {
                familyExistMap.forEach((k, v) -> {
                    if (familyMap.containsKey(k)) {
                        CustomerFamilyMemberInfo cfm = familyMap.get(k);
                        if (!v.getRowsStatus().equals(cfm.getRowsStatus())) {
                            v.setRowsStatus(cfm.getRowsStatus());
                            updateList.add(v);
                        }

                        familyMap.remove(k);
                    }
                });
            }

            Set<CustomerFamilyMemberInfo> saveList = new HashSet<>();
            List<String> patHisNoList = new ArrayList<>();
            familyMap.forEach((k, v) -> {
                saveList.add(v);
                patHisNoList.add(v.getPatHisNo());
            });
            
            // 保存用户数据
            if (CollUtil.isNotEmpty(custUserSaveList)) {
                customerDwdPatientBizMapper.saveHisPatientUser(custUserSaveList);
            }
            // 保存第三方用户数据
            if (CollUtil.isNotEmpty(thirdUserSaveList)) {
                customerDwdPatientBizMapper.saveHisPatientThirdUser(thirdUserSaveList);
            }
            // 更新第三方用户数据
            if (CollUtil.isNotEmpty(thirdUserUpdateList)) {
                // customerThirdUserInfoService.updateBatchById(thirdUserUpdateList);
                customerDwdPatientBizMapper.updateHisPatientThirdUser(thirdUserUpdateList);
            }
            // 保存家属信息
            if (CollUtil.isNotEmpty(saveList)) {
                // 根据同步过来的患者编号[patHisNo] 获取 dwd主患者表id
                Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patHisNoList);
                saveList.forEach(i -> i.setMainPersonId(patientMap.get(i.getPatHisNo())));
                customerDwdPatientBizMapper.saveHisPatientFamilyUser(new ArrayList<>(saveList));
            }
            // 更新家属信息（更新家属绑定关系：解绑/绑定）
            if (CollUtil.isNotEmpty(updateList)) {
                customerDwdPatientBizMapper.updateHisPatientFamilyUser(new ArrayList<>(updateList));
            }
        }

        return true;
    }

    // 原关系：0-本人，1-父母，2-子女，3-夫妻，4-亲属，5-朋友，6-其他
    // 整理后的关系：1:本人 2:配偶 3:父母 4:子女 5:他人
    private static String relationType(String relationId) {
        if (StrUtil.isBlank(relationId)) {
            return null;
        }

        if ("0".equals(relationId)) {
            return "1";
        } else if ("1".equals(relationId)) {
            return "3";
        } else if ("2".equals(relationId)) {
            return "4";
        } else if ("3".equals(relationId)) {
            return "2";
        } else {
            return "5";
        }
    }

    @Override
    public void setSelfRelationTask() {
        customerDwdPatientBizMapper.setSelfRelationTask();
    }

}
