package com.xxh.cloud.customer.framework.modules.task.schedule;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.xxh.cloud.customer.framework.modules.biz.service.HisCustomerUserBizService;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisPatientDTO;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class HisCustomerUserSyncTask {

    @Autowired
    private HisCustomerUserBizService hisCustomerUserBizService;
    
    // 全量同步互联网医院家属信息
    @XxlJob("syncAllHisPatient")
    public void syncAllHisPatient() throws Exception {
        log.info("全量同步互联网医院患者信息开始 >>>>>> ");
        XxlJobHelper.log("全量同步互联网医院患者信息开始 >>>>>> ");
        
        // String jobParam = "2024-05-02,2024-05-21";
        String jobParam = XxlJobHelper.getJobParam();
        if (StrUtil.isBlank(jobParam)) {
            return;
        }

        HisPatientDTO patientDTO = new HisPatientDTO();
        patientDTO.setPage(1);
        patientDTO.setRows(1000);

        String[] dateArray = jobParam.split(",");
        patientDTO.setStartTime(DateUtil.parseDateTime(dateArray[0].trim()).toString());
        if (dateArray.length == 2) {
            patientDTO.setEndTime(DateUtil.parseDateTime(dateArray[1].trim()).toString());
        }

        while (true) {
            // TODO 处理同步的患者信息 1.查询unionid对应的患者账号是否存在，存在则保存，不存在则过滤
            // 若要在同步患者数据时，没有的患者注册账号
            Boolean b = hisCustomerUserBizService.syncHisPatient(patientDTO);
            if (!b) {
                break;
            }
            patientDTO.setPage(patientDTO.getPage() + 1);
        }
        
        XxlJobHelper.log("全量同步互联网医院患者信息结束 <<<<<< ");
        log.info("全量同步互联网医院患者信息结束 <<<<<< ");
    }
    
    // 增量同步互联网医院家属数据
    @XxlJob("syncHisPatient")
    public void syncHisPatient() throws Exception {
        log.info("增量同步互联网医院患者信息开始 >>>>>> ");
        XxlJobHelper.log("增量同步互联网医院患者信息开始 >>>>>> ");
        
        // String jobParam = "10"; // 设定同步多长时间的数据，默认10分钟
        String jobParam = XxlJobHelper.getJobParam();
        if (StrUtil.isBlank(jobParam)) {
            jobParam = "10";
        }

        HisPatientDTO patientDTO = new HisPatientDTO();
        patientDTO.setPage(1);
        patientDTO.setRows(1000);

        Date now = DateUtil.date();
        patientDTO.setStartTime(DateUtil.offsetMinute(now, - Integer.valueOf(jobParam)).toString());
        patientDTO.setEndTime(now.toString());

        while (true) {
            // TODO 处理同步的患者信息 1.查询unionid对应的患者账号是否存在，存在则保存，不存在则过滤
            // 若要在同步患者数据时，没有的患者注册账号
            Boolean b = hisCustomerUserBizService.syncHisPatient(patientDTO);
            if (!b) {
                break;
            }
            patientDTO.setPage(patientDTO.getPage() + 1);
        }

        XxlJobHelper.log("增量同步互联网医院患者信息结束 <<<<<< ");
        log.info("增量同步互联网医院患者信息结束 <<<<<< ");
    }
    
    // 定时任务设置只有一个且未设置家属关系的家属为本人
    @XxlJob("setSelfRelationTask")
    public void setSelfRelationTask() throws Exception {
        log.info("设置本人关系任务开始 >>>>>> ");
        XxlJobHelper.log("设置本人关系任务开始 >>>>>> ");
        
        hisCustomerUserBizService.setSelfRelationTask();

        XxlJobHelper.log("设置本人关系任务结束 <<<<<< ");
        log.info("设置本人关系任务结束 <<<<<< ");
    }
}
