package com.xxh.cloud.log.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.base.entity.LoginLogInfo;
import com.xxh.cloud.log.framework.modules.base.service.LoginLogInfoService;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogQueryDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.LoginLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import com.xxh.cloud.log.framework.modules.biz.utils.convert.LoginLogConvert;
import com.xxh.cloud.log.framework.modules.biz.utils.helper.OperationLogHelper;

import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Log4j2
@Service
public class LoginLogInfoBizServiceImpl implements LoginLogInfoBizService {

    @Autowired
    private LoginLogInfoService loginLogInfoService;


    @Override
    public PageData<LoginLogInfoVO> query(LoginLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<LoginLogInfo> page= loginLogInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<LoginLogInfo>()
                        .eq("rows_status", YN.YES.getCode())
//                        .eq(StrUtil.isNotEmpty(params.getUserId()),"user_id",params.getUserId())
//                        .eq(StrUtil.isNotEmpty(params.getUserName()),"user_name",params.getUserName())
//                        .eq(StrUtil.isNotEmpty(params.getModuleType()),"module_type",params.getModuleType())
//                        .eq(StrUtil.isNotEmpty(params.getObjectType()),"object_type",params.getObjectType())
//                        .eq(StrUtil.isNotEmpty(params.getObjectId()),"object_id",params.getObjectId())
//                        .eq(StrUtil.isNotEmpty(params.getOpType()),"op_type",params.getOpType())
//                        .ge(StrUtil.isNotEmpty(params.getOpDateFrom()),"op_date",params.getOpDateFrom())
//                        .le(StrUtil.isNotEmpty(params.getOpDateTo()),"op_date",params.getOpDateTo())
                        .orderByDesc("create_date")
        );

        PageData<LoginLogInfoVO> data= new PageConverter<LoginLogInfo, LoginLogInfoVO>() {
            @Override
            public LoginLogInfoVO converter(LoginLogInfo entity) {
                return LoginLogConvert.getLoginLogInfoVO(entity);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public LoginLogInfoVO get(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        //todo:
        return null;
    }

    @Override
    public void del(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        loginLogInfoService.update(
                new UpdateWrapper<LoginLogInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .set("update_date",new Date())
                        .set("updater",loginUser==null?"systen":loginUser.getId())
                        .eq("id",id)
        );
    }

    @Override
    public void delBatch(List<String> ids) {
        if(CollUtil.isEmpty(ids)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        loginLogInfoService.update(
                new UpdateWrapper<LoginLogInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .set("update_date",new Date())
                        .set("updater",loginUser==null?"systen":loginUser.getId())
                        .in("id",ids)
        );
    }

    @Override
    public void add(LoginLogAddDTO params) {
        LoginLogInfo entity = new LoginLogInfo();
        entity.setId(OperationLogHelper.getLoginLogId());
        entity.setUserId(params.getUserId());
        entity.setUserName(params.getUserName());
        entity.setOpType(params.getOpType());
        entity.setOpWay(params.getOpWay());
        entity.setOpIp(params.getOpIp());
        entity.setOpPlatform(params.getOpPlatform());
        entity.setOpUserAgent(params.getOpUserAgent());
        entity.setRowsStatus(YN.YES.getCode());
        loginLogInfoService.save(entity);
    }


}
