package com.xxh.cloud.log.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.base.entity.OperationLogInfo;
import com.xxh.cloud.log.framework.modules.base.service.OperationLogInfoService;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.OperationLogAddBatchDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.OperationLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.OperationLogQueryDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.OperationLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.OperationLogInfoBizService;
import com.xxh.cloud.log.framework.modules.biz.utils.convert.OperationLogConvert;
import com.xxh.cloud.log.framework.modules.biz.utils.helper.OperationLogHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Log4j2
@Service
public class OperationLogInfoBizServiceImpl implements OperationLogInfoBizService {

    @Autowired
    private OperationLogInfoService operationLogInfoService;



    @Override
    public PageData<OperationLogInfoVO> query(OperationLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<OperationLogInfo> page= operationLogInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<OperationLogInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getUserId()),"user_id",params.getUserId())
                        .eq(StrUtil.isNotEmpty(params.getUserName()),"user_name",params.getUserName())
                        .eq(StrUtil.isNotEmpty(params.getModuleType()),"module_type",params.getModuleType())
                        .eq(StrUtil.isNotEmpty(params.getObjectType()),"object_type",params.getObjectType())
                        .eq(StrUtil.isNotEmpty(params.getObjectId()),"object_id",params.getObjectId())
                        .eq(StrUtil.isNotEmpty(params.getOpType()),"op_type",params.getOpType())
                        .ge(StrUtil.isNotEmpty(params.getOpDateFrom()),"op_date",params.getOpDateFrom())
                        .le(StrUtil.isNotEmpty(params.getOpDateTo()),"op_date",params.getOpDateTo())
                        .orderByDesc("create_date")
        );

        PageData<OperationLogInfoVO> data= new PageConverter<OperationLogInfo, OperationLogInfoVO>() {
            @Override
            public OperationLogInfoVO converter(OperationLogInfo entity) {
                return OperationLogConvert.getOperationLogInfoVO(entity);
            }
        }.getPageData(page);
        return data;
    }


    @Override
    public void add(OperationLogAddDTO params) {
        ValidatorUtils.validateEntity(params);
        OperationLogInfo entity= OperationLogHelper.getOperationLogInfo_Add(params);
        operationLogInfoService.save(entity);
    }

    @Override
    public void addBatch(OperationLogAddBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        //todo:
    }

    @Override
    public OperationLogInfoVO get(String id) {
        if(StrUtil.isEmpty(id)){
            new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR.getCode(), "ID不能为空");
        }
        return null;
    }

    @Override
    public void del(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        operationLogInfoService.update(
                new UpdateWrapper<OperationLogInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .set("update_date",new Date())
                        .set("updater",loginUser==null?"systen":loginUser.getId())
                        .eq("id",id)
        );
    }

    @Override
    public void delBatch(List<String> ids) {
        if(CollUtil.isEmpty(ids)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        operationLogInfoService.update(
                new UpdateWrapper<OperationLogInfo>()
                        .set("rows_status",YN.NO.getCode())
                        .set("update_date",new Date())
                        .set("updater",loginUser==null?"systen":loginUser.getId())
                        .in("id",ids)
        );
    }


}
