package com.xxh.cloud.log.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.*;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.ErrorLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.LoginLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.OperationLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.OperationLogInfoBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/sys/log/operation")
@Api(tags="系统业务日志管理接口")
public class OperationLogManagerController {

    @Autowired
    private OperationLogInfoBizService operationLogInfoBizService;


    @PostMapping("query")
    @ApiOperation(value = "查询" ,response = OperationLogInfoVO.class)
    @SaCheckPermission("sys:log:operation:query")
    public Result<PageData<OperationLogInfoVO>> query(@RequestBody OperationLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<OperationLogInfoVO> data= operationLogInfoBizService.query(params);
        return new Result<PageData<OperationLogInfoVO>>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:log:operation:info")
    public Result<OperationLogInfoVO> info(@RequestBody OperationLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        OperationLogInfoVO data=operationLogInfoBizService.get(params.getId());
        return new Result<OperationLogInfoVO>().ok(data);
    }

    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:log:operation:del")
    public Result<String> del(@RequestBody OperationLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        operationLogInfoBizService.del(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("delBatch")
    @ApiOperation("批量删除")
    @SaCheckPermission("sys:log:operation:delBatch")
    public Result<String> delBatch(@RequestBody OperationLogGetBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        operationLogInfoBizService.delBatch(params.getIds());
        return new Result<String>().ok("ok");
    }










}
