package com.xxh.cloud.store.manager.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;

import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.OssAliConfigBO;
import com.xxh.cloud.framework.common.param.bos.OssLocalFileConfigBO;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import com.xxh.cloud.store.framework.modules.base.entity.dos.StoreFileInfo;
import com.xxh.cloud.store.framework.modules.base.entity.dto.StoreFileDTO;
import com.xxh.cloud.store.framework.modules.base.entity.vo.OssModel;
import com.xxh.cloud.store.framework.modules.base.service.StoreFileInfoService;
import com.xxh.cloud.store.framework.modules.config.StoreConfig;
import com.xxh.cloud.store.framework.modules.constant.OssErrorEnum;
import com.xxh.cloud.store.framework.modules.service.AliyunCloudStorageService;
import com.xxh.cloud.store.framework.modules.service.LocalFileStorageService;
import com.xxh.cloud.store.framework.modules.utils.StoreFileNameUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

@SaCheckLogin
//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/store/io")
@Api(tags="文件上传服务端模式")
public class FileStoreUploadController {


    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private AliyunCloudStorageService aliyunCloudStorageService;
    @Autowired
    private LocalFileStorageService localFileStorageService;
    @Autowired
    private StoreFileInfoService storeFileInfoService;
    @Autowired
    private StoreConfig storeConfig;

    @PostMapping("/up2AliOss")
    @ApiOperation("上传到阿里云OSS")
    public Result<OssModel> up2AliOss(@RequestParam("file") MultipartFile file) {
        // 获取oss配置数据
        OssAliConfigBO ossAliConfig = paramsClientService.getParamConfig(storeConfig.getAliOss(), ParamValTypeEnum.Ali_Oss.getCode(), new OssAliConfigBO());
        if(file==null || file.isEmpty()) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR.getCode(), "文件不能为空");
        }
        if(file.getSize()>10*1024*1024) {
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR.getCode(), "文件太大");
        }
        String fileFormat= StoreFileNameUtils.getExtension(file.getOriginalFilename());
        String fileName=StoreFileNameUtils.getFileName(StoreFileNameUtils.getImgNo(),fileFormat);
        String url;
        try {
            url = aliyunCloudStorageService.upload(ossAliConfig, file.getInputStream(), fileName);
        } catch (IOException e) {
            e.printStackTrace();
            throw new BusinessException(OssErrorEnum.Oss_Upload_File_Error);
        }
        // 数据库操作
        StoreFileInfo storeFileInfo = storeFileInfoService.add(new StoreFileDTO(storeConfig.getAliOss(), url, fileName, fileFormat));
        return new Result<OssModel>().ok(new OssModel(storeFileInfo));
    }

    @PostMapping("/up2AliLocal")
    @ApiOperation("上传到服务器")
    public Result<OssModel> up2AliLocal(@RequestParam("file") MultipartFile file) {
        // 获取oss配置数据
        OssLocalFileConfigBO ossLocalFileConfigBO =
                paramsClientService.getParamConfig(storeConfig.getFileLocal(), ParamValTypeEnum.File_local_Oss.getCode(), new OssLocalFileConfigBO());
        String fileFormat = StoreFileNameUtils.getExtension(file.getOriginalFilename());
        String fileName=StoreFileNameUtils.getFileName(StoreFileNameUtils.getImgNo(),fileFormat);
        String url;
        try {
            url = localFileStorageService.upload(ossLocalFileConfigBO, file.getInputStream(), fileName);
        } catch (IOException e) {
            e.printStackTrace();
            throw new BusinessException(OssErrorEnum.Oss_Upload_File_Error);
        }
        // 数据库操作
        StoreFileInfo storeFileInfo = storeFileInfoService.add(new StoreFileDTO(storeConfig.getFileLocal(), url, fileName, fileFormat));
        return new Result<OssModel>().ok(new OssModel(storeFileInfo));
    }
    
    @PostMapping("/up2AliLocal/batch")
    @ApiOperation("上传到服务器")
    public Result<List<OssModel>> up2AliLocalMulti(@RequestParam("files") MultipartFile[] files) {
        // 获取oss配置数据
        OssLocalFileConfigBO ossLocalFileConfigBO =
                paramsClientService.getParamConfig(storeConfig.getFileLocal(), ParamValTypeEnum.File_local_Oss.getCode(), new OssLocalFileConfigBO());
        
        List<OssModel> storeFileInfoList = new ArrayList<>(); 
        for (MultipartFile file : files) {
            String fileFormat = StoreFileNameUtils.getExtension(file.getOriginalFilename());
            String fileName=StoreFileNameUtils.getFileName(StoreFileNameUtils.getImgNo(),fileFormat);
            String url;
            try {
                url = localFileStorageService.upload(ossLocalFileConfigBO, file.getInputStream(), fileName);
            } catch (IOException e) {
                e.printStackTrace();
                throw new BusinessException(OssErrorEnum.Oss_Upload_File_Error);
            }
            // 数据库操作
            StoreFileInfo storeFileInfo = storeFileInfoService.add(new StoreFileDTO(storeConfig.getFileLocal(), url, fileName, fileFormat));

            storeFileInfoList.add(new OssModel(storeFileInfo));
        }
        return new Result<List<OssModel>>().ok(storeFileInfoList);
    }
    
    @GetMapping(value = "/get4AliLocal/{fileName}", produces = {MediaType.IMAGE_JPEG_VALUE, MediaType.IMAGE_GIF_VALUE, MediaType.IMAGE_PNG_VALUE})
    @ResponseBody
    @ApiOperation("从服务器下载")
    public byte[] get4AliLocal(HttpServletResponse response, @PathVariable("fileName") String fileName) throws IOException {
        // 获取oss配置数据
        OssLocalFileConfigBO ossLocalFileConfigBO = paramsClientService.getParamConfig(storeConfig.getFileLocal(), ParamValTypeEnum.File_local_Oss.getCode(), new OssLocalFileConfigBO());

        byte[] bytes = null;
        try(FileInputStream inputStream = new FileInputStream(ossLocalFileConfigBO.getLocalPath() + fileName)) {
            bytes = new byte[inputStream.available()];
            inputStream.read(bytes, 0, inputStream.available());
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return bytes;
    }
}
