package com.xxh.cloud.third.framework.modules.thirdWX.helper;


import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.company.cloud.sdk.weixin.mini.model.Code2SessionResp;
import com.company.cloud.sdk.weixin.mp.model.*;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgResp;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoReqModel;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoResp;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisAccessTokenModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdWX.dto.SendTempMsgDTO;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;


@Slf4j
@Component
public class WeixinApiHelper {
    // 调用微信端获取accessToken
    private static String getAccessToken = "/api/weixin/getAccessToken";

    // 用AccessToken查询JsapiTicket
    private static String jsapiTicketUrl = "/api/weixin/jsapiTicket";

    // 小程序端-获取登录信息
    private static String getLoginAuthTokenMiniUrl = "/api/weixin/mini/getLoginAuthToken";

    // 小程序端-获取手机号
    private static String getMobileByOpenIdMiniUrl = "/api/weixin/mini/getMobileByOpenId";

    // 公众号-获取登录信息
    private static String getLoginAuthTokenMpUrl = "/api/weixin/mp/getLoginAuthToken";

    // 获取Unionid
    private static String getUnionidUrl = "/api/weixin/getUnionid";

    // 刷新网页授权调用凭证
    private static String refreshTokenUrl = "/api/weixin/page/refreshToken";

    // 发送模板消息
    private static String sendTempMsgUrl = "/api/weixin/sendTempMsg";

    /**调用微信端获取accessToken */
    public static RespBaseThird<HisAccessTokenModel> getAccessToken(HealthyFrontendConfig config, WxApiConfig wxApi) {
        String bodyJson = JSONObject.toJSONString(wxApi);
        String result = HttpRequest.post(config.getHost() + getAccessToken)
                .body(bodyJson)
                .execute().body();
        log.info("调用微信端获取accessToken----" + result);
        return new RespBaseThird<>(result, HisAccessTokenModel.class);
    }

    /**用AccessToken查询JsapiTicket */
    public static RespBaseThird<JsapiTicketResp> jsapiTicket(HealthyFrontendConfig config, String accessToken) {
        Map<String, String> params = new HashMap<>();
        params.put("accessToken", accessToken);
        String bodyJson = JSONObject.toJSONString(params);
        String result = HttpRequest.post(config.getHost() + jsapiTicketUrl)
                .body(bodyJson)
                .execute().body();
        log.info("用AccessToken查询JsapiTicket----" + result);
        return new RespBaseThird<>(result, JsapiTicketResp.class);
    }

    /**小程序端-获取登录信息 */
    public static RespBaseThird<Code2SessionResp> getLoginAuthTokenMini(HealthyFrontendConfig config, Code2AccessTokenModel params) {
        String bodyJson = JSONObject.toJSONString(params);
        String result = HttpRequest.post(config.getHost() + getLoginAuthTokenMiniUrl)
                .body(bodyJson)
                .execute().body();
        log.info("小程序端-获取登录信息----" + result);
        return new RespBaseThird<>(result, Code2SessionResp.class);
    }

    /**公众号-获取登录信息 */
    public static RespBaseThird<Code2AccessTokenResp> getLoginAuthTokenMp(HealthyFrontendConfig config, Code2AccessTokenModel params) {
        String bodyJson = JSONObject.toJSONString(params);
        String result = HttpRequest.post(config.getHost() + getLoginAuthTokenMpUrl)
                .body(bodyJson)
                .execute().body();
        log.info("公众号-获取登录信息----" + result);
        return new RespBaseThird<>(result, Code2AccessTokenResp.class);
    }

    /**获取unionid */
    public static RespBaseThird<UserBasicInfoResp> getUnionid(HealthyFrontendConfig config, UserBasicInfoReqModel params) {
        String bodyJson = JSONObject.toJSONString(params);
        String result = HttpRequest.post(config.getHost() + getUnionidUrl)
                .body(bodyJson)
                .execute().body();
        log.info("获取unionid----" + result);
        return new RespBaseThird<>(result, UserBasicInfoResp.class);
    }

    /**刷新页面授权的token */
    public static RespBaseThird<RefreshTokenResp> refreshTokenPage(HealthyFrontendConfig config, RefreshTokenModel params) {
        String bodyJson = JSONObject.toJSONString(params);
        String result = HttpRequest.post(config.getHost() + refreshTokenUrl)
                .body(bodyJson)
                .execute().body();
        log.info("刷新页面授权的token----" + result);
        return new RespBaseThird<>(result, RefreshTokenResp.class);
    }

    /**发送模板消息 */
    public static RespBaseThird<SendTempMsgResp> sendTempMsg(HealthyFrontendConfig config, SendTempMsgDTO params) {
        String bodyJson = JSONObject.toJSONString(params);
        // String result = HttpRequest.post(config.getHost() + sendTempMsgUrl)
        //         .body(bodyJson)
        //         .execute().body();
        String result = loopPostHttp(config.getHost() + sendTempMsgUrl, bodyJson, 0);
        log.info("模板消息----" + result);
        return new RespBaseThird<>(result, SendTempMsgResp.class);
    }

    // 调用失败则暂停5秒重新请求，最多重调5次
    private static String loopPostHttp(String url, String body, Integer loopNum) {
        try {
            return HttpRequest.post(url).body(body).execute().body();
        } catch (Exception e) {
            log.info("POST调用异常: {} ", url);
            e.printStackTrace();
            
            // 最多循环5次
            if (loopNum < 5) {
                // 先暂停
                log.info("休眠5000ms");
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                // 出现异常再次调用
                return loopPostHttp(url, body, loopNum + 1);
            }
        }
        return null;
    }
}
