package com.xxh.cloud.third.framework.modules.thirdWX.service;

import cn.hutool.core.util.StrUtil;
import com.company.cloud.sdk.weixin.common.model.WxJSSDKSignModel;
import com.company.cloud.sdk.weixin.common.utils.WxJSSDKSignUtils;
import com.company.cloud.sdk.weixin.mp.model.*;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoReqModel;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.HisApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisAccessTokenModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.bos.ThirdUnifyUserLoginBO;
import com.xxh.cloud.third.framework.modules.thirdWX.constant.XXWeiXinErrorEnum;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.ThirdUserLoginWeixinConvert;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinApiHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class ThirdWeixinLoginMpService {


    @Autowired
    private ThirdWeixinPageAccessTokenService thirdWeixinPageAccessTokenService;
    @Autowired
    private ThirdWeixinAccessTokenService thirdWeixinAccessTokenService;
    @Autowired
    private ParamsClientService paramsClientService;

    /**
     * openId 和 网页授权access_token
     * */
    public ThirdUnifyUserLoginBO getLoginAuthToken(WxApiConfig config, String jscode){
        // 调用api获取openid
        Code2AccessTokenModel code2AccessTokenModel = new Code2AccessTokenModel();
        code2AccessTokenModel.setAppid(config.getAppid());
        code2AccessTokenModel.setSecret(config.getSecret());
        code2AccessTokenModel.setCode(jscode);
//        Code2AccessTokenResp resp = WxMpApi.code2Ticket(code2AccessTokenModel); 调用微信的方法

        // 改成调用三方系统
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        RespBaseThird<Code2AccessTokenResp> respBaseThird = WeixinApiHelper.getLoginAuthTokenMp(healthyFrontend, code2AccessTokenModel);
        Code2AccessTokenResp resp = respBaseThird.getData();
        if (resp == null) {
            throw new BusinessException(XXWeiXinErrorEnum.WX_MP_OpenId);
        }
        thirdWeixinPageAccessTokenService.setPageAccessToken2RefreshToken(config.getAppid(),resp.getOpenid(),resp.getAccess_token(),resp.getExpires_in(),resp.getRefresh_token());

        // 调用his获取accessToken
        RespBaseThird<HisAccessTokenModel> acResp = HisApiHelper.getAccessToken(healthyFrontend);
        // 目前调用微信端获取accessToken
//        RespBaseThird<HisAccessTokenModel> acResp = WeixinApiHelper.getAccessToken(healthyFrontend, config);

        UserBasicInfoResp userBasicInfoResp = new UserBasicInfoResp();
        if (acResp.getData() != null) {
            // 获取unionid
            UserBasicInfoReqModel userBasicInfoReqModel = new UserBasicInfoReqModel();
            userBasicInfoReqModel.setAccess_token(acResp.getData().getAccToken());
            userBasicInfoReqModel.setOpenid(resp.getOpenid());
            RespBaseThird<UserBasicInfoResp> unionidResp = WeixinApiHelper.getUnionid(healthyFrontend, userBasicInfoReqModel);
            if (unionidResp.getData() != null) {
                userBasicInfoResp = unionidResp.getData();
            }
        }

        // 获取unionid
//        UserBasicInfoReqModel userBasicInfoReqModel = new UserBasicInfoReqModel();
//        userBasicInfoReqModel.setAccess_token(accessTokenResp.getAccess_token());
////        userBasicInfoReqModel.setAccess_token(accessToken);
//        userBasicInfoReqModel.setOpenid(resp.getOpenid());
//        UserBasicInfoResp userBasicInfoResp = WxMpUserManageApi.getUserInfo(userBasicInfoReqModel);
//        UserBasicInfoResp userBasicInfoResp = new UserBasicInfoResp();
//        userBasicInfoResp.setUnionid("o4qwy6mjNw74hcRLI17CGg_yJYtI");
//        log.debug("------------------userBasicInfoResp------------------------------");
//        log.debug(JSONObject.toJSONString(userBasicInfoResp));
        return ThirdUserLoginWeixinConvert.getThirdUnifyUserLoginBO(resp, userBasicInfoResp);
    }

    // 获取用户基本信息(UnionID机制)
    public UserBasicInfoResp getUserUnionId(WxApiConfig config, HealthyFrontendConfig healthyFrontend, String accessToken, String openid) {
        if (StrUtil.isBlank(openid)) {
            return null;
        }
        if (null == healthyFrontend) {
            healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        }
        if (StrUtil.isBlank(accessToken)) {
            accessToken = thirdWeixinAccessTokenService.getAccessToken(config);
            if (StrUtil.isBlank(accessToken)) {
                return null;
            }
        }
        // 获取unionid
        UserBasicInfoReqModel userBasicInfoReqModel = new UserBasicInfoReqModel();
        userBasicInfoReqModel.setAccess_token(accessToken);
        userBasicInfoReqModel.setOpenid(openid);
        RespBaseThird<UserBasicInfoResp> unionidResp = WeixinApiHelper.getUnionid(healthyFrontend, userBasicInfoReqModel);
        if (null == unionidResp || unionidResp.getData() == null) {
            return null;
        }

        return unionidResp.getData();
    }


    /**
     * 拉取用户信息(需scope为 snsapi_userinfo)
     * */



    /**
     * 微信js签名
     * */
    public WxJSSDKSignModel wxSign(WxApiConfig config, String url){
        String jsApiTicket=thirdWeixinAccessTokenService.getJsapiToken(config);
        if(StrUtil.isEmpty(jsApiTicket)){
            throw new BusinessException(XXWeiXinErrorEnum.WX_MP_JsApiTicket);
        }
        WxJSSDKSignModel wxResp= WxJSSDKSignUtils.sign(config.getAppid(),url,jsApiTicket);
        return wxResp;
    }











}
