package com.xxh.cloud.user.sys.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaIgnore;

import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.utils.PasswordCheckUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.LogLoginOutTypeEnum;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import com.xxh.cloud.user.framework.modules.base.biz.UserInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserMenuSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.cache.service.RoleMenuCacheService;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdateBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdatePasswordBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@SaCheckLogin
@RestController
@RequestMapping("/app/sys/user")
@Api(tags="运营端用户(服务),用户服务")
public class UserAppController {


    @Autowired
    private UserInfoBizService userInfoBizService;


    @Autowired
    private UserPermissionService userPermissionService;

    @Autowired
    private RoleMenuCacheService roleMenuCacheService;
    @Autowired
    private LoginLogInfoBizService loginLogInfoBizService;


    @PostMapping("getMenu")
    @ApiOperation("获取当前用户菜单")
    public Result<List<MenuVO>> getMenu() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        List<MenuVO> data= userPermissionService.getMenuTreeByUser(loginUser.getId());
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("getMenuList")
    @ApiOperation("获取当前用户菜单列表")
    public Result<List<MenuVO>> getMenuList(@RequestBody UserMenuSearchDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        List<MenuVO> data= userPermissionService.getMenuList(loginUser.getId(), params);
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("getInfoBySelf")
    @ApiOperation("获取当前用户信息")
    public Result<UserVO> getInfoBySelf() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        System.out.println("+++++++++++++++++++++++++++++ "+ JSON.toJSONString(loginUser));
        UserVO data= userInfoBizService.infoById(loginUser.getId());
        return new Result<UserVO>().ok(data);
    }

    @PostMapping("updateBySelf")
    @ApiOperation("修改当前用户信息")
    public Result<UserVO> updateBySelf(@RequestBody UserUpdateBySelfDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        UserVO data= userInfoBizService.update(params, loginUser);
        return new Result<UserVO>().ok(data);
    }

    @PostMapping("updatePasswordBySelf")
    @ApiOperation("修改当前用户密码")
    public Result<UserVO> updatePasswordBySelf(@RequestBody UserUpdatePasswordBySelfDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        PasswordCheckUtils.validatePassword(params.getOldPassWord());
        PasswordCheckUtils.validatePassword(params.getNewPassWord());
        UserVO data= userInfoBizService.changePassword(params, loginUser);
        return new Result<UserVO>().ok(data);
    }


    /**
     * 医生端与管理端注销
     * @return
     */
    @SaIgnore
    @PostMapping("logout")
    @ApiOperation("注销当前用户")
    public Result<String> logout() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        if (null != loginUser) {
            // AuthContextUtils.logout(loginUser.getId());
            AuthContextUtils.logoutByToken(loginUser.getToken());
            // roleMenuCacheService.removeCacheMenuTree(loginUser.getId());
            
            // 登录日志
            LoginLogAddDTO loginLogAddDTO = new LoginLogAddDTO();
            loginLogAddDTO.setUserId(loginUser.getId());
            loginLogAddDTO.setUserName(loginUser.getUserName());
            loginLogAddDTO.setOpType(LogLoginOutTypeEnum.LL_Logout.getCode());
            loginLogAddDTO.setOpWay("登出");
            loginLogAddDTO.setOpIp(IpUtils.getIpAddr());
            loginLogAddDTO.setOpUserAgent(HttpContextUtils.getRequest().getHeader("User-Agent"));
            loginLogAddDTO.setOpPlatform("sys");
            loginLogInfoBizService.add(loginLogAddDTO);
        }

        return new Result<String>().ok("logout ok");
    }






}
