package com.xxh.cloud.user.sys.application.controller;


import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.MD5;

import com.alibaba.fastjson2.JSON;
import com.company.cloud.sdk.weixin.common.model.WxJSSDKSignModel;
import com.company.cloud.wx.sdk.work.inner.WxWorkInnerApi;
import com.company.cloud.wx.sdk.work.inner.model.Authorize2CodeModel;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.utils.PasswordCheckUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.LogLoginOutTypeEnum;
import com.xxh.cloud.log.bean.constant.OperationTypeEnum;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import com.xxh.cloud.third.framework.modules.captcha.entity.dto.ImageCaptchaCreateDTO;
import com.xxh.cloud.third.framework.modules.captcha.entity.vos.ImageCaptchaVO;
import com.xxh.cloud.third.framework.modules.captcha.service.ImageCaptchaService;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.bos.ThirdUnifyUserLoginBO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.dto.ThirdUnifyJsapiSignDTO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.dto.ThirdUnifyUserLoginDTO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.service.ThirdUnifyUserLoginFactoryService;
import com.xxh.cloud.user.framework.modules.base.service.ThirdUserService;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserBindThirdDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserLoginThirdDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserPasswordLoginDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginService;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginThirdService;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import com.xxh.cloud.user.framework.modules.register.service.UserRegisterService;
import com.xxh.cloud.web.common.service.StpUserPermissionCacheService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

@Slf4j
@SaIgnore
@RestController
@RequestMapping("/app/sys/userlogin")
@Api(tags="运营端用户(服务),用户登录服务")
public class UserLoginAppController {

    @Autowired
    private ImageCaptchaService imageCaptchaService;
    @Autowired
    private UserLoginService userLoginService;
    @Autowired
    private UserRegisterService userRegisterService;
    @Autowired
    private UserPermissionService userPermissionService;
    @Autowired
    private StpUserPermissionCacheService stpUserPermissionCacheService;
    @Autowired
    private ThirdUnifyUserLoginFactoryService thirdUnifyUserLoginFactoryService;
    @Autowired
    private UserLoginThirdService userLoginThirdService;
    @Autowired
    private ThirdUserService thirdUserService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private LoginLogInfoBizService loginLogInfoBizService;

    /**
     * 医生端与管理端登录
     * 
     * @param params
     * @return
     */
    @PostMapping("loginByNamePassword")
    @ApiOperation("用户密码验证码登录")
    public Result<UserLoginVO> loginByNamePassword(@RequestBody UserPasswordLoginDTO params) {
        ValidatorUtils.validateEntity(params);
        PasswordCheckUtils.validatePassword(params.getPassword());
        //todo:关闭验证码
        imageCaptchaService.validate(params.getCaptchaVerification(), params.getWordVCode());
        UserLoginVO data= userLoginService.loginBySys(params);
        Set<String> permissions= userPermissionService.getPermissionListByUser(data.getId());
        if (CollUtil.isNotEmpty(permissions)) {
            stpUserPermissionCacheService.addUserPermission(data.getId(),permissions);
        }
        
        // 登录日志
        LoginLogAddDTO loginLogAddDTO = new LoginLogAddDTO();
        loginLogAddDTO.setUserId(data.getId());
        loginLogAddDTO.setUserName(data.getUserName());
        loginLogAddDTO.setOpType(LogLoginOutTypeEnum.LL_Login.getCode());
        loginLogAddDTO.setOpWay("账号");
        loginLogAddDTO.setOpIp(IpUtils.getIpAddr());
        loginLogAddDTO.setOpUserAgent(HttpContextUtils.getRequest().getHeader("User-Agent"));
        loginLogAddDTO.setOpPlatform("sys");
        loginLogInfoBizService.add(loginLogAddDTO);

        return new Result<UserLoginVO>().ok(data);
    }


    @PostMapping("getImageWordCaptcha")
    @ApiOperation("获取图片验证码")
    public Result<ImageCaptchaVO> getImageWordCaptcha(@RequestBody ImageCaptchaCreateDTO params) {
        params.setType("CircleCaptcha");
        params.setNum(20);
        params.setCacheExpire(300);
        ImageCaptchaVO data= imageCaptchaService.createImageCaptcha(params);
        return new Result<ImageCaptchaVO>().ok(data);
    }

    // 测试
    // @GetMapping("/oauth2")
    // @ApiOperation(value = "授权接口", response = String.class)
    // public void oauth2(HttpServletResponse response) {
    //     WxApiConfig wxConfig= paramsClientService.getParamConfig("WX_WORK_1", new WxApiConfig());
    //     try {
    //         Authorize2CodeModel authorize2CodeModel = new Authorize2CodeModel();
    //         authorize2CodeModel.setAppId(wxConfig.getAppid());
    //         authorize2CodeModel.setRedirectUri(wxConfig.getRedirectUri());
    //         authorize2CodeModel.setResponseType("code");
    //         authorize2CodeModel.setScope(wxConfig.getScope());
    //         authorize2CodeModel.setState("WX_WORK_1");
    //         authorize2CodeModel.setAgentId(wxConfig.getAgentid());
    //         response.sendRedirect(WxWorkInnerApi.authorize2Code(authorize2CodeModel));
    //     } catch (IOException e) {
    //         // TODO Auto-generated catch block
    //         e.printStackTrace();
    //     }
    // }

    // // 由企业微信重定向调用
    // @GetMapping("/oauth2/login")
    // @ApiOperation(value = "授权登录接口", response = String.class)
    // public Result<UserLoginVO> oauth2Login(UserLoginThirdDTO params) {
    //     params.setParamsId(params.getState());
    //     return this.login(params);
    // }

    // 企业微信登录
    @PostMapping("login")
    @ApiOperation("三方登录")
    public Result<UserLoginVO> login(@RequestBody UserLoginThirdDTO params) {
        log.info("第三方登录信息 >>>>>> {}", JSON.toJSONString(params));
        ValidatorUtils.validateEntity(params);
        // customerUserValidateService.checkThirdLogin();
        ThirdUnifyUserLoginBO thirdUnifyUserLogin= thirdUnifyUserLoginFactoryService.login(new ThirdUnifyUserLoginDTO(params.getParamsId(),params.getCode()));
        log.info("获取第三方登录信息 >>>>>> {}", JSON.toJSONString(thirdUnifyUserLogin));
        if (null == thirdUnifyUserLogin || StrUtil.isBlank(thirdUnifyUserLogin.getOpenid())) {
            throw new BusinessException("非企业微信用户，登录失败");
        }

        params.setOpenId(thirdUnifyUserLogin.getOpenid());

        UserLoginVO user= userLoginThirdService.login(params);
        log.info("获取登录用户信息 >>>>>> {}", JSON.toJSONString(user));

        if (StrUtil.isNotBlank(user.getId()) && !"0".equals(user.getId())) {
            Set<String> permissions= userPermissionService.getPermissionListByUser(user.getId());
            if (CollUtil.isNotEmpty(permissions)) {
                stpUserPermissionCacheService.addUserPermission(user.getId(),permissions);
            }
        }
        
        // 未绑定用户则返回绑定用户key用于绑定，已绑定过则返回正常token
        if (StrUtil.isBlank(user.getId()) || "0".equals(user.getId())) {
            String cacheKey=StrUtil.format("userKey:{}", MD5.create().digestHex(user.getThirdId()));
            redisUtils.set(cacheKey, user.getThirdId(), 300);
            user.setToken(cacheKey);
        } else {
            String token= AuthContextUtils.login(user.getCompanyNo(),user.getDeptId(),user.getId(),user.getUserName(),user.getThirdId());
            user.setToken(token);
        }
        
        // 登录日志
        LoginLogAddDTO loginLogAddDTO = new LoginLogAddDTO();
        loginLogAddDTO.setUserId(user.getId());
        // loginLogAddDTO.setUserName(user.getUserName());
        loginLogAddDTO.setUserName(user.getThirdOpenId());
        loginLogAddDTO.setOpType(LogLoginOutTypeEnum.LL_Login.getCode());
        loginLogAddDTO.setOpWay("微信");
        loginLogAddDTO.setOpIp(IpUtils.getIpAddr());
        loginLogAddDTO.setOpUserAgent(HttpContextUtils.getRequest().getHeader("User-Agent"));
        loginLogAddDTO.setOpPlatform("sys");
        loginLogInfoBizService.add(loginLogAddDTO);

        return new Result<UserLoginVO>().ok(user);
    }
    
    // 企业登录用户绑定用户信息
    @PostMapping("bind") 
    @ApiOperation("三方登录绑定用户信息")
    public Result<UserLoginVO> bind(@RequestBody UserBindThirdDTO params) {
        ValidatorUtils.validateEntity(params);
        // LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        // log.info("获取登录用户信息 >>>>>> {}", JSON.toJSONString(loginUser));

        Object thirdUserId = redisUtils.get(params.getThirdKey());
        if (null == thirdUserId) {
            throw new BusinessException("请重新授权绑定");
        }
        
        UserLoginVO user = userLoginThirdService.bind(thirdUserId.toString(), params);

        if (null == user) {
            throw new BusinessException("绑定失败");
        }
        
        if (StrUtil.isNotBlank(user.getId()) && !"0".equals(user.getId())) {
            String token = AuthContextUtils.login(user.getCompanyNo(),user.getDeptId(),user.getId(),user.getUserName(),user.getThirdId());
            user.setToken(token);
            Set<String> permissions= userPermissionService.getPermissionListByUser(user.getId());
            if (CollUtil.isNotEmpty(permissions)) {
                stpUserPermissionCacheService.addUserPermission(user.getId(),permissions);
            }
        }
        
        return new Result<UserLoginVO>().ok(user);
    }

    // 分享
    @SaIgnore
    @PostMapping("jsapiSign") 
    @ApiOperation("获取jsapi_ticket签名")
    public Result<WxJSSDKSignModel> jsapiSign(@RequestBody ThirdUnifyJsapiSignDTO params) {
        ValidatorUtils.validateEntity(params);
        // LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        // log.info("获取登录用户信息 >>>>>> {}", JSON.toJSONString(loginUser));

        WxJSSDKSignModel data = thirdUnifyUserLoginFactoryService.jsapiSign(params);

        return new Result<WxJSSDKSignModel>().ok(data);
    }

}
