package com.xxh.cloud.user.sys.application.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.utils.PasswordCheckUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.LogLoginOutTypeEnum;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserPasswordLoginDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserWecomAuthorDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginService;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import com.xxh.cloud.web.common.service.StpUserPermissionCacheService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Set;

@Slf4j
@SaIgnore
@RestController
@RequestMapping("/app/sys/wbmd")
@Api(tags="运营端用户(服务),企业微信授权登陆(WBMD)")
public class UserLoginWbmdController {

    @Autowired
    private UserLoginService userLoginService;
    @Autowired
    private UserPermissionService userPermissionService;
    @Autowired
    private StpUserPermissionCacheService stpUserPermissionCacheService;
    @Autowired
    private LoginLogInfoBizService loginLogInfoBizService;

    @PostMapping("/loginWecomAuthor")
    @ApiOperation("企业微信授权登陆")
    public Result<UserLoginVO> loginWecomAuthor(@RequestBody UserWecomAuthorDTO params) {
        ValidatorUtils.validateEntity(params);
        UserLoginVO data= userLoginService.loginByWbmdOa(params.getCode());
        if (null == data) {
            // 企业微信授权登陆异常返回空对象
            return new Result<UserLoginVO>().success();
        }

        Set<String> permissions= userPermissionService.getPermissionListByUser(data.getId());
        if (CollUtil.isNotEmpty(permissions)) {
            stpUserPermissionCacheService.addUserPermission(data.getId(),permissions);
        }

        // 登录日志
        LoginLogAddDTO loginLogAddDTO = new LoginLogAddDTO();
        loginLogAddDTO.setUserId(data.getId());
        loginLogAddDTO.setUserName(data.getUserName());
        loginLogAddDTO.setOpType(LogLoginOutTypeEnum.LL_Login.getCode());
        loginLogAddDTO.setOpWay("账号");
        loginLogAddDTO.setOpIp(IpUtils.getIpAddr());
        loginLogAddDTO.setOpUserAgent(HttpContextUtils.getRequest().getHeader("User-Agent"));
        loginLogAddDTO.setOpPlatform("sys");
        loginLogInfoBizService.add(loginLogAddDTO);

        return new Result<UserLoginVO>().ok(data);
    }




}
