package com.xxh.cloud.user.framework.modules.base.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.CompanyClientService;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Dept;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptQueryPageDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptTreeDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptUpdateDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.helper.DeptConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.cache.service.DeptCacheService;
import com.xxh.cloud.user.framework.modules.constant.XXUserErrorEnum;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class DeptBizServiceImpl implements DeptBizService {

    @Autowired
    private DeptService deptService;
    @Autowired
    private UserService userService;
    @Autowired
    private DeptCacheService deptCacheService;


    @Autowired
    private CompanyClientService companyClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public List<DeptVO> getTree(DeptTreeDTO params) {
        List<Dept> list= deptService.list(new QueryWrapper<Dept>()
                .eq("company_no",params.getCompanyNo())
                .orderByDesc("pid","sort")
        );

        if(CollUtil.isEmpty(list)){
            return null;
        }

        List<DeptVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(DeptConvertHelper.getDeptVO(item));
        });
        List<DeptVO> data= TreeUtils.build(dataList);
        return data;
    }


    @Override
    public DeptVO add(DeptAddDTO params) {
        ValidatorUtils.validateEntity(params);
        Dept entity=new Dept();
        entity.setId(StrUtil.isNotEmpty(params.getId())? params.getId() : IdUtil.getSnowflake(1,3).nextIdStr());
        entity.setCompanyNo(params.getCompanyNo());
        entity.setName(params.getName());
        entity.setSort(params.getSort());
        entity.setPid(params.getPid());
        entity.setRemark(params.getRemark());
        entity.setManagerUserIds(StrJoinSplitUtils.join(params.getManagerUserIds()));
        entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));

        // 扩展字段
        entity.setDeptType(params.getDeptType());
        entity.setExtend1(params.getExtend1());
        entity.setExtend2(params.getExtend2());
        entity.setExtend3(params.getExtend3());
        entity.setExtend4(params.getExtend4());
        entity.setExtend5(params.getExtend5());
        deptService.save(entity);
        return DeptConvertHelper.getDeptVO(entity);
    }

    @Override
    public DeptVO update(DeptUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        Dept entity=deptService.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(StrUtil.isNotEmpty(params.getName())){
            entity.setName(params.getName());
        }
        if(params.getSort()!=null){
            entity.setSort(params.getSort());
        }
        entity.setRemark(params.getRemark());
        entity.setManagerUserIds(StrJoinSplitUtils.join(params.getManagerUserIds()));

        if (CollectionUtil.isEmpty(params.getDiseaseIds())) {
            entity.setDiseaseIds("");
        } else {
            entity.setDiseaseIds(StrJoinSplitUtils.joinPrefix(params.getDiseaseIds()));
        }

        // 扩展字段
        entity.setDeptType(params.getDeptType());
        entity.setExtend1(params.getExtend1());
        entity.setExtend2(params.getExtend2());
        entity.setExtend3(params.getExtend3());
        entity.setExtend4(params.getExtend4());
        entity.setExtend5(params.getExtend5());
        deptService.updateById(entity);
        deptCacheService.removeById(entity.getId());
        return DeptConvertHelper.getDeptVO(entity);
    }

    @Override
    public DeptVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Dept entity=deptService.getById(id);
        if(entity==null){
            // throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
            return null;
        }
        DeptVO data= DeptConvertHelper.getDeptVO(entity);
        if(StrUtil.isNotEmpty(entity.getManagerUserIds())){
            Set<String> userIds=  StrJoinSplitUtils.split2Set(entity.getManagerUserIds());
            List<AdminUserBO> userList= adminUserClientService.getUserListByIds(userIds);
            data.setManagerUserList(userList);
        }
        return data;
    }



    @Override
    public List<DeptVO> list(DeptSearchDTO params) {
        List<DeptVO> data=new ArrayList<>();
        List<Dept> list= deptService.list(new QueryWrapper<Dept>()
                .eq(StrUtil.isNotEmpty(params.getCompanyNo()), "company_no", params.getCompanyNo())
                .in(CollUtil.isNotEmpty(params.getCompanyNos()),"company_no", params.getCompanyNos())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getPid()), "pid", params.getPid())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .like(StrUtil.isNotEmpty(params.getKeyword()), "name", params.getKeyword())
                .orderByDesc("company_no","sort")
        );
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(DeptConvertHelper.getDeptVO(item));
            });
        }
        return data;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Dept entity=deptService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        long childNum= deptService.count(new QueryWrapper<Dept>().eq("pid",id));
        if(childNum>0){
            throw new BusinessException(XXUserErrorEnum.Dept_Child_Exists);
        }

        long userNum=userService.count(new QueryWrapper<User>().eq("dept_id",id));
        if(userNum > 0){
            throw new BusinessException(XXUserErrorEnum.Dept_User_Exists);
        }
        deptService.removeById(id);

        deptCacheService.removeById(entity.getId());
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<DeptAddDTO> params) {
        log.info("1.同步部门>>>>>> {}", params.size());
        if (CollectionUtil.isEmpty(params)) {
            return;
        }

        // 查询要保存的数据是否存在
        Set<String> ids = new HashSet<>();
        params.forEach(i -> {
            ids.add(i.getId());
        });
        DeptSearchDTO deptSearchDTO = new DeptSearchDTO();
        deptSearchDTO.setIds(new ArrayList<>(ids));
        List<DeptVO> existList = this.list(deptSearchDTO);
        log.info("2.同步时要更新的部门数量>>>>>> {}", existList.size());
        Map<String, DeptVO> existMap = existList.stream().collect(Collectors.toMap(DeptVO::getId, i -> i));
        
        List<Dept> addList = new ArrayList<>(); // 待新增的信息
        List<Dept> updateList = new ArrayList<>(); // 待更新的信息
        params.forEach(i -> {
            DeptVO vo = existMap.get(i.getId());
            Dept entity = new Dept();
            entity.setId(i.getId()); // 部门code=id;
            entity.setCompanyNo(i.getCompanyNo());
            entity.setName(i.getName());
            if (StrUtil.isEmpty(i.getPid()) && vo != null) {
                entity.setPid(vo.getPid()); // 参数中没有pid,保持原有的pid
            } else {
                entity.setPid(i.getPid());
            }
            entity.setSort(i.getSort());
            entity.setRemark(i.getRemark());
            entity.setDeptType(i.getDeptType());
            entity.setExtend1(i.getExtend1());
            entity.setExtend2(i.getExtend2());
            entity.setExtend3(i.getExtend3());
            entity.setExtend5(i.getExtend5());
            if (vo == null) {
                addList.add(entity);
            } else {
                updateList.add(entity);
            }
        });

        log.info("3.新增的部门数量>>>>>> {}", addList.size());
        if (CollectionUtil.isNotEmpty(addList)) {
            deptService.saveOrUpdateBatch(addList);
        }
        log.info("4.更新的部门数量>>>>>> {}", updateList.size());
        if (CollectionUtil.isNotEmpty(updateList)) {
            deptService.updateBatchById(updateList);
        }
    }


    @Override
    public PageData<DeptVO> query(DeptQueryPageDTO params) {
        IPage<Dept> page = deptService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<Dept>()
                .eq(StrUtil.isNotEmpty(params.getCompanyNo()), "company_no", params.getCompanyNo())
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .in(CollUtil.isNotEmpty(params.getIds()), "id", params.getIds())
                .eq(StrUtil.isNotBlank(params.getPid()), "pid", params.getPid())
                .like(StrUtil.isNotEmpty(params.getKeyword()), "name", params.getKeyword())
                .orderByDesc("company_no","sort")
        );
        if (null == page || CollUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        PageData<DeptVO> data = new PageConverter<Dept, DeptVO>() {
            @Override
            public DeptVO converter(Dept entity) {
                return DeptConvertHelper.getDeptVO(entity);
            }
        }.getPageData(page);
        return data;
    }
}
