package com.xxh.cloud.user.framework.modules.base.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.GroupInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.GroupInfo;
import com.xxh.cloud.user.framework.modules.base.entity.dos.GroupUserInfo;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.GroupInfoEditDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.GroupInfoQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.GroupInfoSaveDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.GroupInfoVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.base.helper.GroupInfoConvertHelper;
import com.xxh.cloud.user.framework.modules.base.helper.UserConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.GroupInfoService;
import com.xxh.cloud.user.framework.modules.base.service.GroupUserInfoService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.cache.keys.UserCacheKeyUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class GroupInfoBizServiceImpl implements GroupInfoBizService {

    @Autowired
    private GroupInfoService groupInfoService;

    @Autowired
    private GroupUserInfoService groupUserInfoService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private UserService userService;



    @Override
    public PageData<GroupInfoVO> query(GroupInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<GroupInfo> page= groupInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<GroupInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .orderByDesc("sort")
        );

//        if(params.isFull()){
//
//        }

        PageData<GroupInfoVO> data= new PageConverter<GroupInfo, GroupInfoVO>() {
            @Override
            public GroupInfoVO converter(GroupInfo entity) {
                return GroupInfoConvertHelper.getGroupInfoVO(entity);
            }
        }.getPageData(page);

        return data;
    }

    @Override
    public GroupInfoVO add(GroupInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        GroupInfo entity =  new GroupInfo();
        entity.setId(params.getId());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        groupInfoService.save(entity);
        return GroupInfoConvertHelper.getGroupInfoVO(entity);
    }

    @Override
    public GroupInfoVO edit(GroupInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        GroupInfo entity=groupInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        groupInfoService.updateById(entity);
        redisUtils.delete(UserCacheKeyUtil.getGroupKey(entity.getId()));
        return GroupInfoConvertHelper.getGroupInfoVO(entity);
    }

    @Override
    public GroupInfoVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"分组ID不能为空");
        }
        GroupInfo entity=groupInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return GroupInfoConvertHelper.getGroupInfoVO(entity);
    }

    @Override
    public GroupInfoVO info2NullById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        GroupInfo entity=groupInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        return GroupInfoConvertHelper.getGroupInfoVO(entity);
    }

    @Override
    public List<GroupInfoVO> list(Collection<String> ids) {
        List<GroupInfo> list= groupInfoService.list(
                new QueryWrapper<GroupInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .in("id",ids)
                        .orderByDesc("sort")
        );
        List<GroupInfoVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(GroupInfoConvertHelper.getGroupInfoVO(item));
        });
        return data;
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"分组ID不能为空");
        }
        GroupInfo entity=groupInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        groupInfoService.updateById(entity);
        redisUtils.delete(UserCacheKeyUtil.getGroupKey(entity.getId()));
    }

    @Override
    public void addGroupUser(String id, List<String> userIds) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"分组ID不能为空");
        }
        List<GroupUserInfo> groupUserList= groupUserInfoService.list(
                new QueryWrapper<GroupUserInfo>()
                        .eq("group_id",id)
                        .in("user_id",userIds)
        );

        List<String> uids=new ArrayList<>();
        if(CollUtil.isNotEmpty(groupUserList)){
            groupUserList.forEach(item->{
                uids.add(item.getUserId());
            });
        }

        List<GroupUserInfo> saveList=new ArrayList<>();
        userIds.forEach(userId->{
            if(!uids.contains(userId)){
                GroupUserInfo entity=new GroupUserInfo();
                entity.setId(IdUtil.getSnowflake(5,2).nextIdStr());
                entity.setGroupId(id);
                entity.setUserId(userId);
                saveList.add(entity);
            }
        });

        if(saveList.size()>0){
            groupUserInfoService.saveBatch(saveList);
        }

    }

    @Override
    public void removeGroupUser(String id, List<String> userIds) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"分组ID不能为空");
        }
        groupUserInfoService.remove(
                new QueryWrapper<GroupUserInfo>()
                        .eq("group_id",id)
                        .in("user_id",userIds)
        );
    }

    @Override
    public List<UserVO> userListByGroup(String id) {

        List<GroupUserInfo> groupUserList= groupUserInfoService.list(
                new QueryWrapper<GroupUserInfo>()
                        .eq("group_id",id)
        );

        if(CollUtil.isEmpty(groupUserList)){
            return null;
        }

        Set<String> userIds=new HashSet<>();
        groupUserList.forEach(item->{
            userIds.add(item.getUserId());
        });

        List<User> list=userService.list(
                new QueryWrapper<User>()
                        .in("id",userIds)
                        .orderByDesc("create_date")
        );

        List<UserVO> data=new ArrayList<>();
        list.forEach(item->{
            data.add(UserConvertHelper.getUserVO(item,false));
        });
        return data;
    }

    @Override
    public Map<String, List<UserVO>> userListMapByGroup(Collection<String> ids) {
        List<GroupUserInfo> groupUserList= groupUserInfoService.list(
                new QueryWrapper<GroupUserInfo>()
                        .eq("group_id",ids)
        );

        Set<String> userIds=new HashSet<>();
        Map<String,String> user2GroupMap=new HashMap<>();
        groupUserList.forEach(item->{
            userIds.add(item.getUserId());
//            user2GroupMap.put(item.getUserId(),item.getGroupId());
        });

        //todo:


        return null;
    }


}
