package com.xxh.cloud.user.framework.modules.base.biz.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Dept;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.model.DepartmentLevel;
import com.xxh.cloud.user.framework.modules.base.entity.model.DepartmentLevelDept;
import com.xxh.cloud.user.framework.modules.base.helper.DepartmentLevelInfoApiHelper;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.third.framework.modules.thirdHis.HisWebApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisGenerateSchemeDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisDeptListModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisSchemeUrlModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdHis.service.ThirdHisService;
import com.xxh.cloud.user.framework.modules.base.biz.HisDeptBizService;
import com.xxh.cloud.user.framework.modules.base.biz.HisDoctorBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.HisDeptInfo;
import com.xxh.cloud.user.framework.modules.base.entity.dto.HisDeptQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.HisDoctorQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.HisDeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.HisDoctorVO;
import com.xxh.cloud.user.framework.modules.base.helper.HisDataConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.HisDeptInfoService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class HisDeptBizServiceImpl implements HisDeptBizService {

    @Autowired
    private HisDeptInfoService hisDeptInfoService;
    @Autowired
    private DeptBizService deptBizService;
    @Autowired
    private HisDoctorBizService hisDoctorBizService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private ThirdHisService thirdHisService;

    @Override
    public PageData<HisDeptVO> page(HisDeptQueryDTO params) {
        
        IPage<HisDeptInfo> page = hisDeptInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<HisDeptInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .like(StrUtil.isNotBlank(params.getName()), "name", params.getName())
                .like(StrUtil.isNotBlank(params.getStandardSubjectName()), "standard_subject_name", params.getStandardSubjectName())
                .and(CollUtil.isNotEmpty(params.getStandardSubjectNames()), i -> {
                    params.getStandardSubjectNames().forEach(j -> {
                        i.or().like("standard_subject_name", j);
                    });
                })
                .orderByAsc("name")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        // 查询科室下医生信息
        List<String> deptIds = new ArrayList<>();
        page.getRecords().forEach(i -> deptIds.add(i.getDeptId()));

        HisDoctorQueryDTO doctorQuery = new HisDoctorQueryDTO();
        doctorQuery.setDeptIds(deptIds);
        List<HisDoctorVO> doctorList = hisDoctorBizService.list(doctorQuery);
        Map<String, List<HisDoctorVO>> doctorMap = new HashMap<>();
        doctorList.forEach(i -> {
            List<HisDoctorVO> l = doctorMap.get(i.getDeptId());
            if (null == l) {
                l = new ArrayList<>();
                doctorMap.put(i.getDeptId(), l);
            }
            l.add(i);
        });

        PageData<HisDeptVO> data = new PageConverter<HisDeptInfo, HisDeptVO>() {
            @Override
            public HisDeptVO converter(HisDeptInfo entity) {
                HisDeptVO vo = HisDataConvertHelper.getHisDeptVO(entity);
                vo.setHisDoctorList(doctorMap.get(vo.getDeptId()));
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<HisDeptVO> list(HisDeptQueryDTO params) {
        
        List<HisDeptInfo> list = hisDeptInfoService.list(new QueryWrapper<HisDeptInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .like(StrUtil.isNotBlank(params.getName()), "name", params.getName())
                .like(StrUtil.isNotBlank(params.getStandardSubjectName()), "standard_subject_name", params.getStandardSubjectName())
                .and(CollUtil.isNotEmpty(params.getStandardSubjectNames()), i -> {
                    params.getStandardSubjectNames().forEach(j -> {
                        i.or().like("standard_subject_name", j);
                    });
                })
                .orderByAsc("name")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        // 查询科室下医生信息
        List<String> deptIds = new ArrayList<>();
        list.forEach(i -> deptIds.add(i.getDeptId()));

        HisDoctorQueryDTO doctorQuery = new HisDoctorQueryDTO();
        doctorQuery.setDeptIds(deptIds);
        List<HisDoctorVO> doctorList = hisDoctorBizService.list(doctorQuery);
        Map<String, List<HisDoctorVO>> doctorMap = new HashMap<>();
        doctorList.forEach(i -> {
            List<HisDoctorVO> l = doctorMap.get(i.getDeptId());
            if (null == l) {
                l = new ArrayList<>();
                doctorMap.put(i.getDeptId(), l);
            }
            l.add(i);
        });

        return list.stream().map(i -> {
            HisDeptVO vo = HisDataConvertHelper.getHisDeptVO(i);
            vo.setHisDoctorList(doctorMap.get(vo.getDeptId()));
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public HisDeptVO detail(HisDeptQueryDTO params) {
        HisDeptInfo entity = hisDeptInfoService.getOne(new QueryWrapper<HisDeptInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .eq(StrUtil.isNotBlank(params.getHisDeptId()), "his_dept_id", params.getHisDeptId())
        );
        
        HisDeptVO vo = HisDataConvertHelper.getHisDeptVO(entity);

        if (null != vo) {
            HisDoctorQueryDTO doctorQuery = new HisDoctorQueryDTO();
            doctorQuery.setDeptId(vo.getDeptId());
            List<HisDoctorVO> doctorList = hisDoctorBizService.list(doctorQuery);
            vo.setHisDoctorList(doctorList);
        }

        return vo;
    }

    @Override
    public void syncHisDept() {
        HealthyFrontendConfig healthyFrontendConfig = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // 查询互联网医院部门信息
        RespBaseThird<HisDeptListModel> hisResp = HisWebApiHelper.getDeptsInfo(healthyFrontendConfig);

        List<HisDeptInfo> syncList = new ArrayList<>();
        if (null != hisResp.getData() && CollUtil.isNotEmpty(hisResp.getData().getDepts())) {
            hisResp.getData().getDepts().forEach(i -> {
                HisDeptInfo dept = new HisDeptInfo();
                dept.setId(StrUtil.format("HD{}", IdUtil.getSnowflake(1,1).nextIdStr()));
                dept.setHospitalId(i.getHospitalId());
                dept.setDeptId(i.getDeptId());
                dept.setHisDeptId(i.getHisDeptId());
                dept.setHisParentId(i.getHisParentId());
                dept.setGradeCode(i.getGradeCode());
                dept.setName(i.getName());
                dept.setNamePinYin(i.getNamePinYin());
                dept.setImgUrl(i.getImgUrl());
                dept.setLocation(i.getLocation());
                dept.setNotice(i.getNotice());
                dept.setDescription(i.getDescription());
                dept.setRemark(i.getRemark());
                dept.setBranchName(i.getBranchName());;
                dept.setIsBranch(i.getIsBranch());
                dept.setAsyncFlag(YN.YES.getCode());
                
                syncList.add(dept);
            });
        }

        Set<String> deptIds = new HashSet<>();
        syncList.forEach(i -> {
            deptIds.add(i.getDeptId());
        });

        Map<String, HisDeptInfo> existMap = new HashMap<>();
        List<HisDeptInfo> unAsyncList = new ArrayList<>(); // 未同步更新的数据
        List<HisDeptInfo> exist = hisDeptInfoService.list(new QueryWrapper<HisDeptInfo>()
                // .in("dept_id", deptIds)
        );
        if (CollUtil.isNotEmpty(exist)) {
            exist.forEach(i -> {
                if (deptIds.contains(i.getDeptId())) {
                    existMap.put(i.getDeptId(), i);
                } else {
                    i.setAsyncFlag(YN.NO.getCode());
                    unAsyncList.add(i);
                }
            });
        }

        List<HisDeptInfo> saveList = new ArrayList<>();
        syncList.forEach(i -> {
            HisDeptInfo entity = existMap.get(i.getDeptId());
            if (null != entity) {
                i.setId(entity.getId());
                i.setStandardSubjectName(entity.getStandardSubjectName());
            }
            saveList.add(i);
        });

        // 保存同步过来的数据
        if (CollUtil.isNotEmpty(saveList)) {
            hisDeptInfoService.saveOrUpdateBatch(saveList);
        }
        
        // 未同步的数据更新同步标记，待人工审查
        if (CollUtil.isNotEmpty(unAsyncList)) {
            hisDeptInfoService.updateBatchById(unAsyncList);
        }
    }

    @Override
    public HisSchemeUrlModel getSchemaUrl(HisDeptQueryDTO params) {
        HisDeptVO dept = this.detail(params);
        if (null == dept) {
            return null;
        }
        HisGenerateSchemeDTO hisGenerateSchemeDTO = new HisGenerateSchemeDTO();
        hisGenerateSchemeDTO.setType("2"); // 挂号排班
        hisGenerateSchemeDTO.setDeptId(dept.getDeptId());
        hisGenerateSchemeDTO.setDeptName(dept.getName());
        hisGenerateSchemeDTO.setHisDeptId(dept.getHisDeptId());
        return thirdHisService.generateWebScheme(hisGenerateSchemeDTO);
    }

    @Override
    public void syncDepartmentLevel() {
        // 同步黑名单
        List<DepartmentLevel> departmentLevels = DepartmentLevelInfoApiHelper.departmentLevelList();
        if (CollUtil.isEmpty(departmentLevels)) {
            return;
        }
        List<DeptAddDTO> list = new ArrayList<>();
        for (DepartmentLevel departmentLevel : departmentLevels) {
            if (CollUtil.isEmpty(departmentLevel.getDeptList())) {
                continue;
            }
            for (DepartmentLevelDept department : departmentLevel.getDeptList()) {
                DeptAddDTO deptAddDTO = new DeptAddDTO();
                deptAddDTO.setId(department.getDeptCode());
                deptAddDTO.setCompanyNo("sys");
                deptAddDTO.setName(department.getDeptName());
                deptAddDTO.setPid("");
                deptAddDTO.setDeptType(department.getDeptType());
                deptAddDTO.setExtend1(department.getDeptName());
                deptAddDTO.setExtend3(department.getAddress());
                if (null != department.getValidState()) {
                    deptAddDTO.setExtend5(department.getValidState());
                }
                list.add(deptAddDTO);
            }
        }
        deptBizService.addBatch(list);

    }

}
