package com.xxh.cloud.user.framework.modules.base.biz.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.third.framework.modules.thirdHis.HisWebApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisGenerateSchemeDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisDoctorListModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisSchemeUrlModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdHis.service.ThirdHisService;
import com.xxh.cloud.user.framework.modules.base.biz.HisDoctorBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.HisDoctorInfo;
import com.xxh.cloud.user.framework.modules.base.entity.dto.HisDoctorQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.HisDoctorVO;
import com.xxh.cloud.user.framework.modules.base.helper.HisDataConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.HisDoctorInfoService;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class HisDoctorBizServiceImpl implements HisDoctorBizService {

    @Autowired
    private HisDoctorInfoService hisDoctorInfoService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private ThirdHisService thirdHisService;

    @Override
    public PageData<HisDoctorVO> page(HisDoctorQueryDTO params) {
        
        IPage<HisDoctorInfo> page = hisDoctorInfoService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<HisDoctorInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .like(StrUtil.isNotBlank(params.getDoctorName()), "doctor_name", params.getDoctorName())
                .and(CollUtil.isNotEmpty(params.getDoctorNames()), i -> {
                    params.getDoctorNames().forEach(j -> {
                        i.or().like("doctor_name", j);
                    });
                })
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()), "dept_id", params.getDeptIds())
                .orderByAsc("doctor_id")
        );

        if (CollectionUtil.isEmpty(page.getRecords())) {
            return new PageData<>(null, 0);
        }

        PageData<HisDoctorVO> data = new PageConverter<HisDoctorInfo, HisDoctorVO>() {
            @Override
            public HisDoctorVO converter(HisDoctorInfo entity) {
                HisDoctorVO vo = HisDataConvertHelper.getHisDoctorVO(entity);
                return vo;
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<HisDoctorVO> list(HisDoctorQueryDTO params) {
        
        List<HisDoctorInfo> list = hisDoctorInfoService.list(new QueryWrapper<HisDoctorInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .like(StrUtil.isNotBlank(params.getDoctorName()), "doctor_name", params.getDoctorName())
                .and(CollUtil.isNotEmpty(params.getDoctorNames()), i -> {
                    params.getDoctorNames().forEach(j -> {
                        i.or().like("doctor_name", j);
                    });
                })
                .eq(StrUtil.isNotBlank(params.getDeptId()), "dept_id", params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()), "dept_id", params.getDeptIds())
                .orderByAsc("doctor_id")
        );

        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        return list.stream().map(HisDataConvertHelper::getHisDoctorVO).collect(Collectors.toList());
    }

    @Override
    public HisDoctorVO detail(HisDoctorQueryDTO params) {
        HisDoctorInfo entity = hisDoctorInfoService.getOne(new QueryWrapper<HisDoctorInfo>()
                .eq(StrUtil.isNotBlank(params.getId()), "id", params.getId())        
                .eq(StrUtil.isNotBlank(params.getDoctorId()), "doctor_id", params.getDoctorId())
                .eq(StrUtil.isNotBlank(params.getHisDoctorId()), "his_doctor_id", params.getHisDoctorId())
        );
        return HisDataConvertHelper.getHisDoctorVO(entity);
    }
    
    @Override
    public void syncHisDoctor() {
        HealthyFrontendConfig healthyFrontendConfig = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // 查询互联网医院部门信息
        RespBaseThird<HisDoctorListModel> hisResp = HisWebApiHelper.getDoctorsInfo(healthyFrontendConfig, "1");

        List<HisDoctorInfo> syncList = new ArrayList<>();
        if (null != hisResp.getData() && CollUtil.isNotEmpty(hisResp.getData().getDoctorsInfoDtos())) {
            hisResp.getData().getDoctorsInfoDtos().forEach(i -> {
                HisDoctorInfo doctor = new HisDoctorInfo();
                doctor.setId(StrUtil.format("HD{}", IdUtil.getSnowflake(1,1).nextIdStr()));
                doctor.setDoctorId(i.getDoctorId());
                doctor.setHisDoctorId(i.getHisDoctorId());
                doctor.setDoctorName(i.getDoctorName());
                doctor.setGender(i.getGender());
                doctor.setDeptName(i.getDeptName());
                doctor.setHeadImg(i.getHeadImg());
                doctor.setLevelId(i.getLevelId());
                doctor.setLevelName(i.getLevelName());
                doctor.setDescription(i.getDescription());
                doctor.setExpert(i.getExpert());
                doctor.setDeptId(i.getDeptId());
                doctor.setSkill(i.getSkill());
                doctor.setContactPhone(i.getContactPhone());
                doctor.setHisDeptId(i.getHisDeptId());
                doctor.setAsyncFlag(YN.YES.getCode());
                
                syncList.add(doctor);
            });
        }

        Set<String> doctorIds = new HashSet<>();
        syncList.forEach(i -> {
            doctorIds.add(i.getDoctorId());
        });

        Map<String, HisDoctorInfo> existMap = new HashMap<>();
        List<HisDoctorInfo> unAsyncList = new ArrayList<>(); // 未同步更新的数据
        // 查询所有已同步过的数据
        List<HisDoctorInfo> exist = hisDoctorInfoService.list(new QueryWrapper<HisDoctorInfo>()
                // .in("doctor_id", doctorIds)
        );
        if (CollUtil.isNotEmpty(exist)) {
            exist.forEach(i -> {
                if (doctorIds.contains(i.getDoctorId())) {
                    existMap.put(i.getDoctorId(), i);
                } else {
                    i.setAsyncFlag(YN.NO.getCode());
                    unAsyncList.add(i);
                }
            });
        }

        List<HisDoctorInfo> saveList = new ArrayList<>();
        syncList.forEach(i -> {
            HisDoctorInfo entity = existMap.get(i.getDoctorId());
            if (null != entity) {
                i.setId(entity.getId());
            }
            saveList.add(i);
        });

        // 保存同步过来的数据
        if (CollUtil.isNotEmpty(saveList)) {
            hisDoctorInfoService.saveOrUpdateBatch(saveList);
        }
        
        // 未同步的数据更新同步标记，待人工审查
        if (CollUtil.isNotEmpty(unAsyncList)) {
            hisDoctorInfoService.updateBatchById(unAsyncList);
        }
    }

    @Override
    public HisSchemeUrlModel getSchemaUrl(HisDoctorQueryDTO params) {
        HisDoctorVO doctor = this.detail(params);
        if (null == doctor) {
            return null;
        }
        HisGenerateSchemeDTO hisGenerateSchemeDTO = new HisGenerateSchemeDTO();
        hisGenerateSchemeDTO.setType("1"); // 医生
        hisGenerateSchemeDTO.setDoctorId(doctor.getDoctorId());
        return thirdHisService.generateWebScheme(hisGenerateSchemeDTO);
    }

}
