package com.xxh.cloud.user.framework.modules.base.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.xxh.cloud.basic.client.api.bean.bos.CompanyBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.helper.DesensitizedHelper;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.bos.PostInfoBO;
import com.xxh.cloud.user.client.api.bean.bos.RoleInfoBO;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleUser;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserUpdateDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.constant.UserStatus;
import com.xxh.cloud.user.framework.modules.login.helper.UserLoginHelper;
import io.swagger.annotations.ApiModelProperty;

import java.util.List;
import java.util.Map;
import java.util.Set;

public class UserConvertHelper {


    /**
     *
     * */
    public static UserVO getUserVO(User entity,boolean isDesensitized){
        UserVO data=new UserVO();
        BeanUtil.copyProperties(entity,data,"postIds","roleIds","personInfo");
        if(isDesensitized){
            data.setUserName(DesensitizedHelper.userName(data.getUserName()));
            data.setRealName(DesensitizedHelper.realName(data.getRealName()));
            data.setMobile(DesensitizedHelper.mobilePhone(data.getMobile()));
            data.setEmail(DesensitizedHelper.email(data.getEmail()));
        }

        if (StrUtil.isNotEmpty(entity.getPersonInfo())) {
            data.setPersonInfo(JSONObject.parseObject(entity.getPersonInfo()));
        }

        if (StrUtil.isNotEmpty(entity.getPostIds())) {
            data.setPostIds(StrJoinSplitUtils.split2Set(entity.getPostIds()));
        }

        return data;
    }



    public static UserVO getUserVO(User entity,boolean isDesensitized ,
                                   Map<String, CompanyBO> companyMap,
                                   Map<String, DeptBO> deptMap,
                                   Map<String, PostInfoBO> postMap,
                                   Map<String, AdminUserBO> opUserMap,
                                   Map<String, RoleInfoBO> roleMap,
                                   Map<String, Set<String>> userRoleMap){
        UserVO data=getUserVO(entity,isDesensitized);

        if(MapUtil.isNotEmpty(companyMap) && StrUtil.isNotEmpty(data.getCompanyNo())){
            CompanyBO company = companyMap.get(data.getCompanyNo());
            data.setCompanyName(company==null?null:company.getName());
        }

        if(MapUtil.isNotEmpty(deptMap) && StrUtil.isNotEmpty(data.getDeptId())){
            DeptBO dept = deptMap.get(data.getDeptId());
            data.setDeptName(dept==null?null:dept.getName());
        }

        if(MapUtil.isNotEmpty(postMap) && CollUtil.isNotEmpty(data.getPostIds())){
            data.getPostIds().forEach(postId->{
                data.addPost(postMap.get(postId));
            });
        }

        if(MapUtil.isNotEmpty(opUserMap)){
            if(StrUtil.isNotEmpty(data.getCreator())){
                AdminUserBO user=  opUserMap.get(data.getCreator());
                data.setCreatorName(user==null?null:user.getRealName());
            }
            if(StrUtil.isNotEmpty(data.getUpdater())){
                AdminUserBO user=  opUserMap.get(data.getUpdater());
                data.setUpdaterName(user==null?null:user.getRealName());
            }
        }

        if(MapUtil.isNotEmpty(roleMap) && MapUtil.isNotEmpty(userRoleMap)){
            Set<String> userRoleSet=  userRoleMap.get(data.getId());
            if(CollUtil.isNotEmpty(userRoleSet)){
                userRoleSet.forEach(roleId->{
                    data.addRole(roleMap.get(roleId));
                });
            }
        }

        return data;
    }


//    /**
//     *
//     * */
//    public static UserVO getUserVO(User entity, boolean isDesensitized, Map<String, RoleVO> roleMap, List<RoleUser> roleUsers){
//        UserVO data=getUserVO(entity,isDesensitized);
//
//        if(CollUtil.isNotEmpty(roleUsers)){
//            roleUsers.forEach(item->{
//                if(data.getId().equals(item.getUserId())){
//                    data.addRole(roleMap.get(item.getRoleId()));
//                }
//            });
//        }
//
//        return data;
//    }




    /**
     *
     * */
    public static User getUserEntity_Add(UserAddDTO params){
        User entity=new User();
        entity.setId(getUserId());
        entity.setCompanyNo(params.getCompanyNo());
        entity.setUserName(params.getUserName());
        entity.setPassWord(UserLoginHelper.getPasswordMd5(params.getPassWord()));
        entity.setMobile(params.getMobile());
        entity.setEmail(params.getEmail());
        entity.setRealName(params.getRealName());
        entity.setHeadUrl(params.getHeadUrl());
        entity.setGender(params.getGender());
        entity.setDeptId(params.getDeptId());
        entity.setIsSuperAdmin(params.getIsSuperAdmin());
        entity.setPostIds(StrJoinSplitUtils.join(params.getPostIds()));
        if(MapUtil.isNotEmpty(params.getPersonInfo())){
            entity.setPersonInfo(JSON.toJSONString(params.getPersonInfo()));
        }
        entity.setStatus(UserStatus.US_Normal.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        entity.setSubjectCode(params.getSubjectCode()); // 202402251353 学科
        entity.setSubjectOrder(params.getSubjectOrder());
        entity.setSubjectRole(params.getSubjectRole());
        return entity;
    }



    /**
     *
     * */
    public static void getUserEntity_Edit(User entity, UserUpdateDTO params){
        entity.setUserName(params.getUserName());
        entity.setMobile(params.getMobile());
        entity.setEmail(params.getEmail());
        entity.setRealName(params.getRealName());
        entity.setHeadUrl(params.getHeadUrl());
        entity.setGender(params.getGender());
        entity.setDeptId(params.getDeptId());
        entity.setIsSuperAdmin(params.getIsSuperAdmin());
        entity.setPostIds(StrJoinSplitUtils.join(params.getPostIds()));
        if(MapUtil.isNotEmpty(params.getPersonInfo())){
            entity.setPersonInfo(JSON.toJSONString(params.getPersonInfo()));
        }
        entity.setSubjectCode(params.getSubjectCode()); // 202402251353 学科
        entity.setSubjectOrder(params.getSubjectOrder());
        entity.setSubjectRole(params.getSubjectRole());
    }


    /**
     *
     * */
    public static RoleUser getRoleUserEntity(User user,String roleId){
        RoleUser entity=new RoleUser();
        entity.setId(IdUtil.getSnowflake(1,2).nextIdStr());
        entity.setUserId(user.getId());
        entity.setRoleId(roleId);
        return entity;
    }


    /**
     *
     * */
    public static String getUserId(){
        return StrUtil.format("UE{}", IdUtil.getSnowflake(1,1).nextIdStr());
    }

    public static String getThirdUserId(){
        return StrUtil.format("UE{}", IdUtil.getSnowflake(1,2).nextIdStr());
    }

}
