package com.xxh.cloud.user.framework.modules.base.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Dept;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Menu;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleMenu;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleUser;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuUpdateDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.helper.DeptConvertHelper;
import com.xxh.cloud.user.framework.modules.base.helper.MenuConvertHelper;
import com.xxh.cloud.user.framework.modules.base.mapper.MenuMapper;
import com.xxh.cloud.user.framework.modules.base.service.MenuService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xxh.cloud.user.framework.modules.base.service.RoleMenuService;
import com.xxh.cloud.user.framework.modules.base.service.RoleUserService;
import com.xxh.cloud.user.framework.modules.constant.XXUserErrorEnum;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 菜单信息表 服务实现类
 */
@Service
public class MenuServiceImpl extends ServiceImpl<MenuMapper, Menu> implements MenuService {


    @Autowired
    private RoleMenuService roleMenuService;
    @Autowired
    private RoleUserService roleUserService;


    @Override
    public List<MenuVO> getTree() {
        List<Menu> list= this.list();
        return this.tree(list,null);
    }


    @Override
    public List<MenuVO> getTreeByRole(String roleId) {
        if(StrUtil.isEmpty(roleId)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"角色ID不能为空");
        }
        List<Menu> list= this.list(new QueryWrapper<Menu>()
                .inSql("id",StrUtil.format("SELECT menu_id  FROM sys_role_menu WHERE role_id='{}'",roleId))
        );
        return this.tree(list,null);
    }

    @Override
    public List<MenuVO> getTreeAllByRole(String roleId) {
        if(StrUtil.isEmpty(roleId)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"角色ID不能为空");
        }
        List<RoleMenu> roleMenuList= roleMenuService.list(new QueryWrapper<RoleMenu>());
        Set<String> menuIds=new HashSet<>();
        if(CollUtil.isNotEmpty(roleMenuList)){
            roleMenuList.forEach(item->{
                menuIds.add(item.getMenuId());
            });
        }
        List<Menu> list= this.list();
        return this.tree(list,menuIds);
    }

    @Override
    public List<MenuVO> getListByRole(MenuSearchDTO params) {
        ValidatorUtils.validateEntity(params);
        List<Menu> list= this.list(new QueryWrapper<Menu>()
                .inSql("id",StrUtil.format("SELECT menu_id  FROM sys_role_menu WHERE role_id='{}'",params.getRoleId()))
        );
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(MenuConvertHelper::getMenuVO).collect(Collectors.toList());
    }

    private List<MenuVO> tree(List<Menu> list, Set<String> roleMenuIds){
        if(CollUtil.isEmpty(list)){
            return null;
        }
        List<MenuVO> dataList=new ArrayList<>();
        list.forEach(item->{
            dataList.add(MenuConvertHelper.getMenuVO(item,roleMenuIds));
        });
        List<MenuVO> data= TreeUtils.build(dataList);
        return data;
    }


    @Override
    public MenuVO add(MenuAddDTO params) {
        ValidatorUtils.validateEntity(params);
        Menu entity = MenuConvertHelper.getMenu(params);
        this.save(entity);
        return MenuConvertHelper.getMenuVO(entity);
    }

    @Override
    public MenuVO update(MenuUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        Menu entity=this.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        MenuConvertHelper.getMenu(entity, params);
        this.updateById(entity);
        return MenuConvertHelper.getMenuVO(entity);
    }

    @Override
    public MenuVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Menu entity=this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return MenuConvertHelper.getMenuVO(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Menu entity=this.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        long childNum= this.count(new QueryWrapper<Menu>().eq("pid",id));
        if(childNum>0){
            throw new BusinessException(XXUserErrorEnum.Menu_Child_Exists);
        }

        List<RoleUser> roleUserList= roleUserService.list(
                new QueryWrapper<RoleUser>()
                        .inSql("role_id",StrUtil.format("SELECT role_id FROM sys_role_menu WHERE menu_id='{}'",id))
        );

        this.removeById(id);

        roleMenuService.remove(new QueryWrapper<RoleMenu>().eq("menu_id",id));

        if(CollUtil.isNotEmpty(roleUserList)){
            for(RoleUser roleUser:roleUserList){
                AuthContextUtils.logout(roleUser.getUserId());
            }
        }

    }
}
