package com.xxh.cloud.user.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.BasicCacheServiceImpl;
import com.xxh.cloud.user.framework.modules.base.biz.PostInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.PostInfo;
import com.xxh.cloud.user.framework.modules.base.entity.vos.PostInfoVO;
import com.xxh.cloud.user.framework.modules.base.helper.PostInfoConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.PostInfoService;
import com.xxh.cloud.user.framework.modules.cache.keys.UserCacheKeyUtil;
import com.xxh.cloud.user.framework.modules.cache.service.PostInfoCacheService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Log4j2
@Service
public class PostInfoCacheServiceImpl extends BasicCacheServiceImpl<PostInfoVO> implements PostInfoCacheService {

    @Autowired
    private PostInfoService postInfoService;


    @Override
    public String getCacheKey(String id) {
        return UserCacheKeyUtil.getPostKey(id);
    }

    @Override
    public String getCacheKeyPattern() {
        return UserCacheKeyUtil.getPostMainKey();
    }

    @Override
    public Class<PostInfoVO> getJsonClazz() {
        return PostInfoVO.class;
    }

    @Override
    public String getId(PostInfoVO entity) {
        return entity.getId();
    }

    @Override
    public PostInfoVO getById2DB(String id) {

        if(StrUtil.isEmpty(id)){
            return null;
        }
        PostInfo entity=postInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            return null;
        }
        return PostInfoConvertHelper.getPostInfoVO(entity);
    }

    @Override
    public List<PostInfoVO> getListByIds2DB(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return null;
        }

        List<PostInfo> list= postInfoService.list(
                new QueryWrapper<PostInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .in("id",ids)
                        .orderByDesc("sort")
        );
        List<PostInfoVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }

        list.forEach(item->{
            data.add(PostInfoConvertHelper.getPostInfoVO(item));
        });
        return data;
    }


}
