package com.xxh.cloud.user.framework.modules.client.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.company.cloud.wx.sdk.work.inner.WxWorkInnerApi;
import com.company.cloud.wx.sdk.work.inner.model.MessageSendResp;
import com.company.cloud.wx.sdk.work.inner.model.MessageSendTextCardModel;
import com.company.cloud.wx.sdk.work.inner.model.MessageSendTextCardModel.TextCard;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.third.framework.modules.sms.model.SmsRequestMessage;
import com.xxh.cloud.third.framework.modules.sms.service.SmsSendService;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdWX.dto.SendWxWorkMsgDTO;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinWorkApiHelper;
import com.xxh.cloud.third.framework.modules.thirdWX.service.ThirdWeixinLoginWorkService;
import com.xxh.cloud.user.client.api.AdminUserThirdClientService;
import com.xxh.cloud.user.client.api.bean.bos.SendWxWorkResultBO;
import com.xxh.cloud.user.client.api.bean.dto.SendWxWorkClientDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dos.ThirdUser;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.service.ThirdUserService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class AdminUserThirdClientServiceImpl implements AdminUserThirdClientService {

    @Autowired
    private ThirdUserService thirdUserService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private ThirdWeixinLoginWorkService thirdWeixinLoginWorkService;
    @Autowired
    private UserService userService;
    @Autowired
    private SmsSendService smsSendService;
 
    @Override
    public SendWxWorkResultBO sendWxWorkMsg(SendWxWorkClientDTO dto) {
        // 查询第三方用户信息
        ThirdUser thirdUser = thirdUserService.getOne(new QueryWrapper<ThirdUser>()
            .eq("user_id", dto.getUserId())
        );

        if (null == thirdUser) {
            return null;
        }
        
        // 查询配置
        WxApiConfig wxConfig= paramsClientService.getParamConfig(thirdUser.getParamsId(), new WxApiConfig());
        log.info("企业微信配置 >>> : {}", JSONObject.toJSONString(wxConfig));
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // 查询accesstoken 目前调用跳板机调用微信端
        String accessToken = thirdWeixinLoginWorkService.getAccessToken(wxConfig);

        SendWxWorkMsgDTO params = new SendWxWorkMsgDTO();
        params.setAccessToken(accessToken);
        params.setOpenid(thirdUser.getOpenId());
        params.setAgentid(wxConfig.getAgentid());
        params.setMsgtype(dto.getMsgtype());
        params.setData(dto.getMsgcontent());

        // 直接调用企业微信发送消息
        // MessageSendResp resp = this.sendWxWorkTextcardMsg(params);
        // return new SendWxWorkResultBO(0, "00", resp.getMsgid());
        
        // 调用前置机发送企业微信消息
        RespBaseThird<MessageSendResp> resp = WeixinWorkApiHelper.sendWxWorkAgentMsg(healthyFrontend, params);
        if (resp.getCode() != 0) {
            log.error(JSONObject.toJSONString(resp));
        }
        return new SendWxWorkResultBO(resp.getData().getErrcode(), resp.getData().getErrmsg(), resp.getData().getMsgid());
    }

    // 直接调用企业微信发送文本卡片消息
    private MessageSendResp sendWxWorkTextcardMsg(SendWxWorkMsgDTO params) {
        
        MessageSendTextCardModel model = new MessageSendTextCardModel();
        model.setAccess_token(params.getAccessToken());
        model.setAgentid(params.getAgentid());
        model.setTouser(params.getOpenid());
        model.setMsgtype("textcard");

        Map<String, String> paramMap = params.getData();
        TextCard textCard = new TextCard();
        textCard.setTitle(paramMap.get("title"));
        textCard.setDescription(paramMap.get("description"));
        textCard.setUrl(paramMap.get("url"));
        textCard.setBtntxt(paramMap.get("btntxt"));
        model.setTextcard(textCard);

        // 发送
        log.info("wechat work request send textcard message >>> : {}", JSONObject.toJSONString(model));
        MessageSendResp resp = WxWorkInnerApi.messageSendTextcard(model);
        log.info("wechat work response send textcard message <<< : {}", JSONObject.toJSONString(resp));
        return resp;
    }

    @Override
    public void sendHisSMSMsg(String userId, String hisSMSMsg) {
        // 查询用户信息
        User user = userService.getById(userId);

        if (null == user || StrUtil.isBlank(user.getMobile())) {
            return;
        }
        
        List<SmsRequestMessage> messages = new ArrayList<>();
        messages.add(new SmsRequestMessage(user.getMobile(), hisSMSMsg));
        smsSendService.sendHisSms(messages);
    }

}
