package com.xxh.cloud.user.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.user.client.api.DeptBizClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Dept;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.helper.DeptConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import com.xxh.cloud.user.framework.modules.client.helper.DeptClientConvertHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class DeptBizClientServiceImpl implements DeptBizClientService {

    @Autowired
    private DeptService deptService;

    @Override
    public List<DeptBO> getDeptTreeList(String id) {
        List<DeptVO> deptTree = this.getDeptTree(id);

        if (CollUtil.isEmpty(deptTree)) {
            return new ArrayList<>(0);
        }

        List<DeptBO> deptBOList = new ArrayList<>();
        this.circle(deptBOList, deptTree);

        return deptBOList;
    }
    
    private List<DeptVO> getDeptTree(String id) {
        List<Dept> list = deptService.list(new QueryWrapper<Dept>()
                .and(i -> i.eq("id", id).or().eq("pid", id))
                .orderByDesc("pid", "sort")
        );

        if(CollUtil.isEmpty(list)){
            return null;
        }

        List<DeptVO> dataList = new ArrayList<>();
        list.forEach(item -> {
            dataList.add(DeptConvertHelper.getDeptVO(item));
        });
        List<DeptVO> data = TreeUtils.build(dataList);
        return data;
    }

    private void circle(List<DeptBO> deptBOList, List<DeptVO> deptTree) {
        deptTree.forEach(i -> {
            deptBOList.add(DeptClientConvertHelper.getDeptBO(i));

            List<DeptVO> childrenList = i.getChildren();
            if (CollUtil.isEmpty(childrenList)) {
                return;
            }
            this.circle(deptBOList, childrenList);
        });
    }

}
