package com.xxh.cloud.user.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.dto.DeptQueryDTO;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.cache.service.DeptCacheService;
import com.xxh.cloud.user.framework.modules.client.helper.DeptClientConvertHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class DeptClientServiceImpl implements DeptClientService {


    @Autowired
    private DeptCacheService deptCacheService;

    @Autowired
    private DeptBizService deptBizService;


    @Override
    public DeptBO getDeptById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        DeptVO dept = deptCacheService.getById(id);
        return DeptClientConvertHelper.getDeptBO(dept);
    }

    @Override
    public List<DeptBO> getDeptListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<DeptVO> list = deptCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(DeptClientConvertHelper::getDeptBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, DeptBO> getDeptMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<DeptVO> list = deptCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(DeptVO::getId, DeptClientConvertHelper::getDeptBO)));
    }

    @Override
    public List<DeptBO> getDeptListByCompany(String companyNo) {
        if(StrUtil.isEmpty(companyNo)){
            return new ArrayList<>();
        }
        List<DeptVO> list = deptBizService.list(new DeptSearchDTO(companyNo));
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(DeptClientConvertHelper::getDeptBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, DeptBO> getDeptMapByCompany(String companyNo) {
        if(StrUtil.isEmpty(companyNo)){
            return new HashMap<>();
        }
        List<DeptVO> list = deptBizService.list(new DeptSearchDTO(companyNo));
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(DeptVO::getId, DeptClientConvertHelper::getDeptBO)));
    }

    @Override
    public List<DeptBO> getDeptList(DeptQueryDTO params) {

        DeptSearchDTO deptSearchDTO = new DeptSearchDTO();
        deptSearchDTO.setCompanyNo(params.getCompanyNo());
        deptSearchDTO.setId(params.getId());
        deptSearchDTO.setKeyword(params.getName());
        deptSearchDTO.setPid(params.getPid());
        
        List<DeptVO> list = deptBizService.list(deptSearchDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(DeptClientConvertHelper::getDeptBO).collect(Collectors.toList());
    }


}
