package com.xxh.cloud.user.framework.modules.login.helper;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;

import java.util.HashMap;
import java.util.Map;

import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.helper.DesensitizedHelper;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.user.framework.modules.base.entity.dos.ThirdUser;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.helper.UserConvertHelper;
import com.xxh.cloud.user.framework.modules.constant.XXUserErrorEnum;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserLoginThirdDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import io.swagger.annotations.ApiModelProperty;

public class UserLoginHelper {


    /**
     *  登录对象
     * */
    public static UserLoginVO getUserLoginVO(User user,boolean isDesensitized){
        Map<String, Object> extraData = new HashMap<>();
        if (StrUtil.isNotEmpty(user.getCompanyNo())) {
            extraData.put("companyNo", user.getCompanyNo());
        }
        if (StrUtil.isNotEmpty(user.getDeptId())) {
            extraData.put("deptId", user.getDeptId());
        }
        if (StrUtil.isNotEmpty(user.getUserName())) {
            extraData.put("userName", user.getUserName());
        }
        if (StrUtil.isNotEmpty(user.getIsSuperAdmin())) {
            extraData.put("isSuperAdmin", user.getIsSuperAdmin());
        }
        String token = AuthContextUtils.login(user.getId(), extraData);
        // String token= AuthContextUtils.login(user.getCompanyNo(),user.getDeptId(),user.getId(),user.getUserName(),null);

        UserLoginVO data=new UserLoginVO();
        BeanUtil.copyProperties(user,data);
        if(isDesensitized){
            data.setUserName(DesensitizedHelper.userName(data.getUserName()));
            data.setRealName(DesensitizedHelper.realName(data.getRealName()));
            data.setMobile(DesensitizedHelper.mobilePhone(data.getMobile()));
            data.setEmail(DesensitizedHelper.email(data.getEmail()));
        }
        data.setToken(token);
        return data;
    }


    /**
     *  校验用户输入的密码
     * */
    public static void checkUserPassword(String userPassword ,String password){
        String reqPassword= UserLoginHelper.getPasswordMd5(password);
        if(!userPassword.equals(reqPassword)){
            throw new BusinessException(XXUserErrorEnum.User_Password_Error);
        }
    }

    /**
     * 密码MD5加密
     * */
    public static String getPasswordMd5(String password){
        return DigestUtil.md5Hex16(password);
    }


    public static UserLoginVO getUserLoginVO(ThirdUser thirdUser, User user){
        UserLoginVO data=getUserLoginVO(thirdUser);
        if(user!=null){
            BeanUtil.copyProperties(user,data);
        }
        return data;
    }

    public static UserLoginVO getUserLoginVO(ThirdUser thirdUser){
        UserLoginVO data=new UserLoginVO();
        data.setThirdId(thirdUser.getId());
        data.setThirdParamsId(thirdUser.getParamsId());
        data.setThirdOpenId(thirdUser.getOpenId());
        data.setThirdUnionId(thirdUser.getUnionId());
        data.setId(thirdUser.getUserId());

        data.setCompanyNo(thirdUser.getCompanyNo());
        return data;
    }

    public static ThirdUser getThirdUserInfo(UserLoginThirdDTO params){
        ThirdUser entity=new ThirdUser();
        entity.setId(UserConvertHelper.getThirdUserId());
        entity.setParamsId(params.getParamsId());
        entity.setOpenId(params.getOpenId());
        entity.setUnionId(params.getUnionId());
        entity.setUserId("0");
        entity.setCompanyNo(params.getCompanyNo());
        entity.setType(params.getType());
        entity.setNickName(params.getNickName());
        entity.setHeadUrl(params.getHeadUrl());
        entity.setSourceOne(params.getSourceOne());
        entity.setSourceTwo(params.getSourceTwo());
        entity.setSourceThree(params.getSourceThree());
        return entity;
    }


}
