package com.xxh.cloud.user.framework.modules.mq.consumer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.xxh.cloud.common.mq.bean.MQDeptBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptAddDTO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 部门数据消费者
 * */
@Log4j2
@Component
@ConditionalOnProperty(name = "consumer.dept.enable", havingValue = "true")
public class DeptConsumer {

    @Autowired
    private DeptBizService deptBizService;

    @JmsListener(destination = HMQQueueName.Que_Dept)
    public void queueDeptData(String message) {
        log.info("1.部门消费者消费消息开始>>>>>> message={}", message);

        if (StrUtil.isBlank(message)) {
            return;
        }
        List<MQDeptBean> mqMessageList = JSONArray.parseArray(message, MQDeptBean.class);

        if (CollectionUtil.isEmpty(mqMessageList)) {
            return;
        }

        List<DeptAddDTO> list = new ArrayList<>();
        mqMessageList.forEach(i -> {
            DeptAddDTO deptAddDTO = new DeptAddDTO();
            deptAddDTO.setId(i.getId());
            deptAddDTO.setCompanyNo("sys");
            deptAddDTO.setName(i.getDeptName());
            deptAddDTO.setPid(StrUtil.isNotBlank(i.getPid()) ? i.getPid() : "0");
            deptAddDTO.setDeptType(i.getDeptType());
            deptAddDTO.setExtend1(i.getDeptName());
            deptAddDTO.setExtend2(String.valueOf(i.getWebDeptId()));
            if (null != i.getStatus()) {
                deptAddDTO.setExtend5(String.valueOf(i.getStatus()));
            }
            list.add(deptAddDTO);
        });


        log.info("2.部门消息信息保存>>>>>> {}", JSON.toJSONString(list));
        deptBizService.addBatch(list);

        log.info("3.部门消费者消费消息结束<<<<<<");
    }


}
