package com.xxh.cloud.user.company.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptInfoDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptTreeDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.DeptUpdateDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/com/user/dept")
@Api(tags="商户端用户(管理),部门管理")
public class DeptComManagerController {


    @Autowired
    private DeptBizService deptBizService;


    @PostMapping("tree")
    @ApiOperation("部门树")
    @SaCheckPermission("com:user:dept:tree")
    public Result<List<DeptVO>> tree() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        DeptTreeDTO params=new DeptTreeDTO();
        params.setCompanyNo(loginUser.getCompanyNo());
        List<DeptVO> data= deptBizService.getTree(params);
        return new Result<List<DeptVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增部门")
    @SaCheckPermission("com:user:dept:add")
    public Result<DeptVO> add(@RequestBody DeptAddDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        params.setCompanyNo(loginUser.getCompanyNo());
        ValidatorUtils.validateEntity(params);
        DeptVO data= deptBizService.add(params);
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改部门")
    @SaCheckPermission("com:user:dept:update")
    public Result<DeptVO> update(@RequestBody DeptUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DeptVO data= deptBizService.update(params);
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取部门")
    @SaCheckPermission("com:user:dept:info")
    public Result<DeptVO> info(@RequestBody DeptInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        DeptVO data=deptBizService.infoById(params.getId());
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除部门")
    @SaCheckPermission("com:user:dept:del")
    public Result<String> del(@RequestBody DeptInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        deptBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }










}
