package com.xxh.cloud.user.sys.manager.controller;

import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.HisDeptBizService;
import com.xxh.cloud.user.framework.modules.base.biz.HisDoctorBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.HisDeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.HisDoctorVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:user:dept")
@RestController
@RequestMapping("/sys/his/data")
@Api(tags="运营端互联网医院管理")
public class HisDataManagerController {

    @Autowired
    private HisDeptBizService hisDeptBizService;
    @Autowired
    private HisDoctorBizService hisDoctorBizService;

    @PostMapping("/dept/query")
    @ApiOperation("部门分页")
    // @SaCheckPermission("sys:his:dept:query")
    public Result<PageData<HisDeptVO>> deptQuery(@RequestBody HisDeptQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<HisDeptVO> data = hisDeptBizService.page(params);
        return new Result<PageData<HisDeptVO>>().ok(data);
    }

    @PostMapping("/dept/list")
    @ApiOperation("部门列表")
    // @SaCheckPermission("sys:his:dept:query")
    public Result<List<HisDeptVO>> deptList(@RequestBody HisDeptQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<HisDeptVO> data = hisDeptBizService.list(params);
        return new Result<List<HisDeptVO>>().ok(data);
    }

    @PostMapping("/dept/info")
    @ApiOperation("获取部门")
    // @SaCheckPermission("sys:his:dept:info")
    public Result<HisDeptVO> info(@RequestBody HisDeptQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        HisDeptVO data=hisDeptBizService.detail(params);
        return new Result<HisDeptVO>().ok(data);
    }

    @PostMapping("/doctor/query")
    @ApiOperation("医生分页")
    // @SaCheckPermission("sys:his:doctor:query")
    public Result<PageData<HisDoctorVO>> query(@RequestBody HisDoctorQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<HisDoctorVO> data = hisDoctorBizService.page(params);
        return new Result<PageData<HisDoctorVO>>().ok(data);
    }

    @PostMapping("/doctor/list")
    @ApiOperation("医生列表")
    // @SaCheckPermission("sys:his:dept:query")
    public Result<List<HisDoctorVO>> doctorList(@RequestBody HisDoctorQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<HisDoctorVO> data = hisDoctorBizService.list(params);
        return new Result<List<HisDoctorVO>>().ok(data);
    }

    @PostMapping("/doctor/info")
    @ApiOperation("获取医生")
    // @SaCheckPermission("sys:his:doctor:info")
    public Result<HisDoctorVO> info(@RequestBody HisDoctorQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        HisDoctorVO data=hisDoctorBizService.detail(params);
        return new Result<HisDoctorVO>().ok(data);
    }

}
