package com.xxh.cloud.user.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.PostInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.PostInfoVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/sys/user/post")
@Api(tags="运营端用户(管理),岗位管理")
public class PostManagerController {

    @Autowired
    private PostInfoBizService postInfoBizService;




    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:user:post:query")
    public Result<PageData<PostInfoVO>> query(@RequestBody PostInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if(StrUtil.isEmpty(params.getCompanyNo())){
            params.setCompanyNo(Constant.SysCompanyNo);
        }
        PageData<PostInfoVO> data= postInfoBizService.query(params);
        return new Result<PageData<PostInfoVO>>().ok(data);
    }


    @PostMapping("list")
    @ApiOperation("列表")
    @SaCheckPermission("sys:user:post:list")
    public Result<List<PostInfoVO>> list(@RequestBody PostInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        if(StrUtil.isEmpty(params.getCompanyNo())){
            params.setCompanyNo(Constant.SysCompanyNo);
        }
        List<PostInfoVO> data= postInfoBizService.list(params);
        return new Result<List<PostInfoVO>>().ok(data);
    }



    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:user:post:add")
    public Result<PostInfoVO> add(@RequestBody PostInfoAddDTO params) {
        ValidatorUtils.validateEntity(params);
        PostInfoVO data= postInfoBizService.add(params);
        return new Result<PostInfoVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:user:post:update")
    public Result<PostInfoVO> update(@RequestBody PostInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        PostInfoVO data= postInfoBizService.edit(params);
        return new Result<PostInfoVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:user:post:info")
    public Result<PostInfoVO> info(@RequestBody PostInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        PostInfoVO data=postInfoBizService.infoById(params.getId());
        return new Result<PostInfoVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:user:post:del")
    public Result<String> del(@RequestBody PostInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        postInfoBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }




}
