package com.xxh.cloud.user.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.utils.PasswordCheckUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.UserInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


//@SaCheckPermission("sys:user:userInfo")
@RestController
@RequestMapping("/sys/user/userInfo")
@Api(tags="运营端用户(管理),用户管理")
public class UserManagerController {


    @Autowired
    private UserInfoBizService userInfoBizService;


    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:user:userInfo:query")
    public Result<PageData<UserVO>> query(@RequestBody UserQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        params.setDesensitize(false);
        PageData<UserVO> data= userInfoBizService.query(params);
        return new Result<PageData<UserVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:user:userInfo:add")
    public Result<UserVO> add(@RequestBody UserAddDTO params) {
        ValidatorUtils.validateEntity(params);
        PasswordCheckUtils.validatePassword(params.getPassWord());
        UserVO data= userInfoBizService.add(params);
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:user:userInfo:update")
    public Result<UserVO> update(@RequestBody UserUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        UserVO data= userInfoBizService.update(params);
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("updateStatus")
    @ApiOperation("修改状态")
    @SaCheckPermission("sys:user:userInfo:updateStatus")
    public Result<UserVO> updateStatus(@RequestBody UserUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        UserVO data= userInfoBizService.updateStatus(params);
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("updatePassword")
    @ApiOperation("修改密码")
    @SaCheckPermission("sys:user:userInfo:updatePassword")
    public Result<UserVO> updatePassword(@RequestBody UserUpdatePasswordDTO params) {
        ValidatorUtils.validateEntity(params);
        PasswordCheckUtils.validatePassword(params.getPassWord());
        UserVO data= userInfoBizService.updatePassword(params);
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:user:userInfo:info")
    public Result<UserVO> info(@RequestBody UserInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        UserVO data=userInfoBizService.infoById(params.getId());
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:user:userInfo:del")
    public Result<String> del(@RequestBody UserInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        userInfoBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }




}
